#include <stdio.h>			/* **/
#include <stdarg.h>			/* **/
#include <string.h>			/* **/
#include <malloc.h>			/* **/
#include <stdlib.h>			/* **/
#include <unistd.h>			/* **/
#include <time.h>			/* **/
#include <errno.h>			/* **/
#include <fcntl.h>			/* **/
#include <termios.h>			/* **/
#include <pwd.h>			/* **/	
#include <sys/wait.h>			/* **/
#include <dirent.h>
#include "ucrp.h"
/* to disable all debug code undefine __UCRP_DEBUG__ macro **/
#define __UCRP_DEBUG__			/* enable debug code **/
#define UCRP_USAGE \
"UCRP %d.%d (c) 2006-2010 Dmitriy I.Cherkashin, www.ucrouter.ru\n"  \
"email: dch@ucrouter.ru , divch@users.sourceforge.net , dch@ucrouter.com\n"  \
"phone: +8-909-936-8809; ICQ: 474-363-900;\n"  \
"This is not public software, you can't use this program without\n"  \
"license agreement, which available via option --license\n"  \
"with noted above author. To see license agreement use:\n"  \
"ucrp --license\n"  \
"  or\n"  \
"ucrp --license-ru\n"  \
"  for russian license translation.\n"
#define UCRP_LICENSE \
"UCRP %d.%d (c) 2006-2010 Dmitriy I.Cherkashin, www.ucrouter.ru\r\n"  \
"email: dch@ucrouter.ru , divch@users.sourceforge.net , dch@ucrouter.com\r\n"  \
"phone: +8-909-936-8809; ICQ: 474-363-900;\r\n"  \
"This program is free software with source codes\r\n"  \
"downloadable via internation communicational net.\r\n"  \
"If you have source codes of this software original or modified,\r\n"  \
"you must provide ability to free receive original source codes\r\n"  \
"of this software to another users even in internation communicational\r\n"  \
"net absense without any unusual additional payment exept\r\n" \
"downloading files from internation communicational net cost or\r\n" \
"mail cost.\r\n"
#define UCRP_LICENSE_RU \
"UCRP %d.%d (c) 2006-2010  ., www.ucrouter.ru\r\n" \
" : dch@ucrouter.ru,divch@users.sourceforge.net,dch@ucrouter.com\r\n"  \
": +8-909-936-8809; ICQ: 474-363-900;\r\n" \
"     \r\n" \
"      \r\n" \
" .\r\n" \
"       \r\n" \
"  ,     \r\n" \
"     \r\n" \
"     \r\n" \
"      \r\n" \
"  ,     \r\n" \
"    \r\n" \
"     .\r\n"
#define UCRP_BS   	     0x7f	/* Back Space Key **/
#define UCRP_DEL 	 0x335b1b	/* Delete Key **/
u_int8_t ucrp_version_hi = 1;		/* program version lo digit */
u_int8_t ucrp_version_lo = 16;		/* program version hight digit */
/* command type **/
#define UCRP_ENCRYPT 	 0x0001		/* encrypt file **/
#define UCRP_DECRYPT 	 0x0002		/* decrypt file **/
#define UCRP_INFO 	 0x0004		/* encrypted file info **/
#define UCRP_CLEAR 	 0x0008		/* clear disk space and history **/
/* exit cmd **/
#define UCRP_LIC	 0x0001 	/* print license agreement & exit **/
#define UCRP_LICRU	 0x0002 	/* print license agreement, ru & exit **/
#define UCRP_HELP	 0x0003 	/* print help usage & exit **/
#define UCRP_VERSION	 0x0004		/* print version & exit **/
#define UCRP_TODO	 0x0005		/* print todo note & exit **/
#define UCRP_CHANGELOG	 0x0006		/* print change log & exit **/
#define UCRP_MAN	 0x0007		/* print man page and exit **/
#define UCRP_UNRELEASED  0x0008		/* unreleased features and bugs **/
#define UCRP_PASSWD_SIZE 	1024
#define UCRP_FILE_NAME_SIZE 	1024
char ucrp_passwd[UCRP_PASSWD_SIZE] = "";
char ucrp_pfile[UCRP_FILE_NAME_SIZE] = ""; /* passwd file **/
u_int8_t ucrp_hide_passwd = 1;		/* hide password, 1 - *, 2 - $, 3 - d */
int ucrp_hack = 5;			/* hack count **/
u_int8_t ucrp_local = 0;		/* take executable files from local **/
/* quiet mode, 1 - no print, 2 - only usage, todo, changelog **/
char ucrp_quiet = 0;			/* **/
u_int8_t ucrp_clear_mode = 0;		/* current dirrectory clear mode **/	
#define UCRP_CLEAR_INC 	0 		/* increment filling **/
#define UCRP_CLEAR_CLR 	1 		/* write 0 **/
#define UCRP_CLEAR_SET 	2 		/* write FF **/
u_int32_t ucrp_cmd = 0;			/* ucrp command */
FILE * ucrp_fi = 0;			/* input file descriptor **/
FILE * ucrp_fo = 0;			/* output file descriptor **/
u_int16_t ucrp_path_size = 256;		/* ucrp path size */
#define UCRP_FILE_NAME_SIZE	1024
char ucrp_ifile[UCRP_FILE_NAME_SIZE] = ""; /* input file name **/
char ucrp_ofile[UCRP_FILE_NAME_SIZE] = ""; /* output file name **/
char ucrp_cdir[UCRP_FILE_NAME_SIZE] = "";  /* current dirrectory **/  
char ucrp_pdir[UCRP_FILE_NAME_SIZE] = "";  /* previous current dirrectory **/  
#define COMP_LEN 8192			/* compability buffer length **/
u_char ucrp_comp_buf[COMP_LEN] = "";
#define __UCRP_XOR_ENABLE__		/* enable XOR algoritm **/
#define __UCRP_ADD_ENABLE__		/* enable ADD algoritm **/
#define __UCRP_ROT_ENABLE__		/* enable ROT algoritm **/
#define __UCRP_TAB_ENABLE__		/* enable TAB algoritm **/
#define __UCRP_ECC_ENABLE__		/* enable ECC algoritm **/
#define __UCRP_BTS_ENABLE__		/* enable BTS {bit swap algorithm} **/
#define __UCRP_BEV_ENABLE__		/* enable BEV {bit even algorithm} **/
#define __UCRP_XRE_ENABLE__		/* enable XRE xor enhanced **/
#ifdef  __UCRP_XOR_ENABLE__
#define UCRP_XOR_100   	64		/* xor compatibility **/
#define UCRP_XOR_118   	71		/* xor compatibility **/
#endif 							/* __UCRP_XOR_ENABLE__ **/
#ifdef  __UCRP_ADD_ENABLE__
#define UCRP_ADD_100   	65		/* add compatibility **/
#define UCRP_ADD_118   	65		/* add compatibility **/
#endif 							/* __UCRP_ADD_ENABLE__ **/
#ifdef  __UCRP_ROT_ENABLE__
#define UCRP_ROT_100   	66		/* byte rotate **/
#define UCRP_ROT_118   	66		/* byte rotate **/
#endif 							/* __UCRP_ROT_ENABLE__ **/
#ifdef  __UCRP_TAB_ENABLE__
#define UCRP_TAB_100   	67		/* byte table **/
#define UCRP_TAB_118   	68		/* byte table **/
#endif 							/* __UCRP_TAB_ENABLE__ **/
#ifdef  __UCRP_ECC_ENABLE__
#define UCRP_ECC_100   	68		/* ecc coding **/
#define UCRP_ECC_118   	69		/* ecc coding **/
#endif  						/* __UCRP_ECC_ENABLE__ **/
#ifdef __UCRP_BTS_ENABLE__
#define UCRP_BTS_100   	69		/* bit swapping algoritm **/
#define UCRP_BTS_118   	67		/* bit swapping algoritm **/
#endif  						/* __UCRP_BTS_ENABLE__ **/
#ifdef __UCRP_BEV_ENABLE__
#define UCRP_BEV_100   	70		/* bit event **/
#define UCRP_BEV_118   	70		/* bit event **/
#endif 							/* __UCRP_BEV_ENABLE__ **/
#ifdef __UCRP_XRE_ENABLE__
#define UCRP_XRE_100    71		/* xor enhanced **/
#define UCRP_XRE_118    64		/* xor enhanced **/
#endif 							/* __UCRP_XRE_ENABLE__ **/
u_int8_t ucrp_xor_comp = UCRP_XOR_118;  /* **/
u_int8_t ucrp_xre_comp = UCRP_XRE_118;	/* **/
u_int8_t ucrp_add_comp = UCRP_ADD_118;	/* **/
u_int8_t ucrp_rot_comp = UCRP_ROT_118;	/* **/
u_int8_t ucrp_bts_comp = UCRP_BTS_118;	/* **/
u_int8_t ucrp_tab_comp = UCRP_TAB_118;	/* **/
u_int8_t ucrp_ecc_comp = UCRP_ECC_118;	/* **/
u_int8_t ucrp_bev_comp = UCRP_BEV_118;	/* **/
#define UCRP_SBL            72		/* source buffer size **/
#define UCRP_SBL_LEN        32		/* source buffer size length **/
#define UCRP_DBL           104		/* destination buffer size **/
#define UCRP_DBL_LEN   	    32		/* destination buffer size length **/
#define KEY_LEN  8192			/* key buffer length **/
u_char ucrp_key_buf[KEY_LEN] = "";
#define UCRP_XOR_BASE              0	/* xor value base **/
#define UCRP_XOR_LEN               8	/* xor value length **/
#define UCRP_ADD_BASE              8	/* add value base **/
#define UCRP_ADD_LEN               8	/* add value legnth **/
#define UCRP_ROT_BASE             16	/* ror value base **/
#define UCRP_ROT_LEN               8	/* ror value length **/
#define UCRP_TAB_BASE_100         24	/* tab value base **/
#define UCRP_TAB_LEN               8	/* tab value length **/
#define UCRP_BTS_BASE_100         32	/* ecc value base **/
#define UCRP_BTS_LEN               8	/* ecc value length **/
#define UCRP_BTS_TABLE_BASE_100   40	/* bts value base {bit swapping} **/
#define UCRP_BTS_TABLE_LEN        64	/* bts value length {bit swapping} **/
#define UCRP_XRL_BASE            104	/* xor mask change value low **/
#define UCRP_XRL_LEN               5	/* xor mask change low length **/
#define UCRP_XRV_BASE            109	/* xor mask change low value base **/
#define UCRP_XRV_LEN               3	/* xor mask change low value length **/
#define UCRP_TABLE_BASE_100     2048 	/* 1024  **/
#define UCRP_TABLE_LEN          1024

#define UCRP_BTS_BASE_118         24	/* ecc value base **/
#define UCRP_TAB_BASE_118         32	/* tab value base **/
#define UCRP_BTS_TABLE_BASE_118 2048	/* bts value base {bit swapping} **/
#define UCRP_TABLE_BASE_118     2112 	/* 2048 + 64 **/
u_int16_t ucrp_bts_base = UCRP_BTS_BASE_118;
u_int16_t ucrp_tab_base = UCRP_TAB_BASE_118;
u_int16_t ucrp_bts_table_base = UCRP_BTS_TABLE_BASE_118;
u_int16_t ucrp_table_base = UCRP_TABLE_BASE_118;
/* low value **/
#define UCRP_LOW_NULL	     0		/* null low **/
#define UCRP_LOW_AD3	     1		/* add low **/
#define UCRP_LOW_ROT	     2		/* rot low **/
#define UCRP_LOW_TAB_100     3		/* tab low **/
#define UCRP_LOW_BTS_100     4		/* bts low **/
#define UCRP_LOW_BTS_118     3		/* bts low **/
#define UCRP_LOW_TAB_118     3		/* tab low **/
u_int8_t ucrp_low_bts = UCRP_LOW_BTS_118;
u_int8_t ucrp_low_tab = UCRP_LOW_TAB_118;
#define UCRP_LOW_MAX	     4		/* low maximun value **/
#define UCRP_MUL_LOW	     5		/* mul low **/
#define UCRP_PWR_LOW         6		/* pwer low **/
#define UCRP_SAR_LOW	     7		/* sar low **/
#define UCRP_BEV_LOW	     8		/* bev low **/
#define UCRP_MLL_LOW	     9		/* mll low **/
#define UCRP_MOV_LOW	    10		/* mov low **/
#define UCRP_LOW_11	    11		/* TODO **/
#define UCRP_LOW_12	    12		/* TODO **/
#define UCRP_LOW_13	    13		/* TODO **/
#define UCRP_LOW_14	    14		/* TODO **/
#define UCRP_LOW_15	    15		/* TODO **/
#define UCRP_LOW_16	    16		/* TODO **/
#define UCRP_LOW_17	    17		/* TODO **/
#define UCRP_LOW_18	    18		/* TODO **/
#define UCRP_LOW_19	    19		/* TODO **/
#define UCRP_LOW_20	    20		/* TODO **/
#define UCRP_LOW_21	    21		/* TODO **/
#define UCRP_LOW_22	    22		/* TODO **/
#define UCRP_LOW_23	    23		/* TODO **/
#define UCRP_LOW_24	    24		/* TODO **/
#define UCRP_LOW_25	    25		/* TODO **/
#define UCRP_LOW_26	    26		/* TODO **/
#define UCRP_LOW_27	    27		/* TODO **/
#define UCRP_LOW_28	    28		/* TODO **/
#define UCRP_LOW_29	    29		/* TODO **/
#define UCRP_LOW_30	    30		/* TODO **/
#define UCRP_LOW_31	    31		/* TODO **/

#define UCRP_SRC_SIZE 0x10000		/* source buffer size **/
#define UCRP_BUF_SIZE (UCRP_SRC_SIZE + ((UCRP_SRC_SIZE / 256) * 3)) /* destination buffer size **/
static u_int32_t ucrp_src_buf_size = UCRP_SRC_SIZE;
static u_int32_t ucrp_dst_buf_size = UCRP_BUF_SIZE;
static u_char * ucrp_src = 0;		/* source buffer **/
static u_char * ucrp_dst = 0;		/* destination buffer **/
#ifdef __UCRP_XOR_ENABLE__		/* **/
#define UCRP_XOR_ENABLE  0x00000001	/* xor enable */
#endif 						/* __UCRP_XOR_ENABLE__ **/
#ifdef __UCRP_ADD_ENABLE__		/* **/
#define UCRP_ADD_ENABLE  0x00000002	/* add enable */
#endif 						/* __UCRP_ADD_ENABLE__ **/
#ifdef __UCRP_ROT_ENABLE__		/* **/
#define UCRP_ROT_ENABLE  0x00000004	/* byte rotate enable */
#endif 						/* __UCRP_ROT_ENABLE__ **/
#ifdef __UCRP_TAB_ENABLE__		/* **/
#define UCRP_TAB_ENABLE  0x00000008	/* table enable */
#endif 						/* __UCRP_TAB_ENABLE__ **/
#ifdef __UCRP_ECC_ENABLE__		/* **/
#define UCRP_ECC_ENABLE  0x00000010	/* ecc enable */
#endif 						/* __UCRP_ECC_ENABLE__ **/
#ifdef __UCRP_BTS_ENABLE__		/* **/
#define UCRP_BTS_ENABLE  0x00000020	/* bit swaping enable */
#endif 						/* __UCRP_BTS_ENABLE__ **/
#ifdef __UCRP_BEV_ENABLE__		/* **/
#define UCRP_BEV_ENABLE  0x00000040	/* bit event enable */
#endif 						/* __UCRP_BEV_ENABLE__ **/
#ifdef __UCRP_XRE_ENABLE__		/* **/
#define UCRP_XRE_ENABLE  0x00000080	/* xor enhanced enable */
#endif 						/* __UCRP_XRE_ENABLE__ **/
static u_int32_t ucrp_enable = (
 0xFFFFFFFF ^ 
 (UCRP_XOR_ENABLE
 )	
);
#define UMSG_PS 		"%s"
#define UMSG_EMP 		""
#define	UMSG_CRNL		"\r\n"
#define	UMSG_PC 		"%c"
#define	UMSG_02X 		"%02x"
#define	UMSG_PRESS_ANY_KEY	"Press any key ..."
#define	UMSG_VERSION 		"\r\nVersion : %d.%d\r\n"
#define UMSG_BS 		"\b"
#define UMSG_SP 		" "
#define UMSG_STDIN_ERROR 	"\r\nBad stdin descriptor\r\n"
#define UMSG_TCGETATTR_STDIN_ERROR "\r\n-tcgetattr stdin error: (%d)\r\n"
#define UMSG_TCSETATTR_STDIN_ERROR "\r\ntcsetattr stdin error : (%d)\r\n"
#define UMSG_SELECT_ERROR	"\r\nselect error : (%d)\r\n"
#define UMSG_STDIN_READ_ERROR	"\r\nstdin read error : (%d)\r\n"
#define UMSG_GET_FCNTL_ERROR 	"Read file {%s} mode error\r\n"
#define UMSG_O_NONBLOCK_ERROR 	"set O_NONBLOCK mode error\r\n"
#define UMSG_RANDOM_FILE_NAME	"/dev/random"
#define UMSG_URANDOM_FILE_NAME	"/dev/urandom"
#define UMSG_INFO_VERSION	"version %s\n"
#define UMSG_INFO_XOR		"xor %s %d\n"
#define UMSG_INFO_XRE		"xre %s %d\n"
#define UMSG_INFO_ON		"on"
#define UMSG_INFO_OFF		"off"
#define UMSG_LOW_NULL		"null"
#define UMSG_LOW_AD3		"add"
#define UMSG_LOW_ROT		"rot"
#define UMSG_LOW_TAB		"tab"
#define UMSG_LOW_BTS		"bts"
#define UMSG_LOW_UNKN		"unkn"
#define UMSG_HASH 		"hash:"
#define UMSG_TTAB		"ttab :"
#define UMSG_BTTB		"bttb :"
#define UMSG_RTAB 		"rtab :"
#define UMSG_INFO_XRL		"xrl %s %d\r\n"
#define UMSG_INFO_XRV   	"xrv %d\r\n"
#define UMSG_INFO_ADD		"add %s %d\r\n"
#define UMSG_INFO_ROT 		"rot %s %d\r\n"
#define UMSG_INFO_TAB 		"tab %s %d\r\n"
#define UMSG_INFO_ECC 		"ecc %s\r\n"
#define UMSG_INFO_BTS 		"bts %s %d\r\n"
#define UMSG_INFO_TTAB 		"translate table:\r\n"
#define UMSG_INFO_SBL 		"source block length %d {0x%x}\r\n"
#define UMSG_INFO_DBL 		"destination block length %d {0x%x}\r\n"
#define UMSG_INFO_TRT 		"translate table:\r\n"
#define UMSG_XOR_ERR  		"xor algs error\r\n"
#define	UMSG_CHANGELOG \
    "\r\nChangeLog\r\n"
extern volatile int kill_link;		/* > 1, exit from select loop **/
extern volatile int ucrp_sig_pipe;	/* **/
extern volatile int ucrp_sig_chld;
extern volatile u_int8_t ucrp_sig_alrm;	/* > 1, exit from select loop **/
extern volatile u_int8_t ucrp_debug_alrm; /* print debug note in alarm handler **/
#if 1
int ucrp_error(const char *frmt, ...) 	/* error message **/
{
 if(ucrp_quiet == 1) {			/* no messages in quiet mode **/
    return(1);
 } else {
    char pbuf[512];			/* message buffer */
    int rvl;				/* return code */
    rvl = 1;				/* return code start value */
    va_list valt;			/* argument list **/
    va_start(valt, frmt);		/* init argument list **/
    u_int32_t cnt;			/* bytes in buffer */
    cnt = vsnprintf(pbuf, sizeof(pbuf), frmt, valt);	/* **/
    if(cnt >= sizeof(pbuf)) {		/* out of buffer check */
	rvl = 0;			/* return code clear */
    }
    va_end(valt);				/* **/
    if(fprintf(stderr, UMSG_PS, pbuf) < 0) {	/* print error message to stderr **/	
	rvl = 0;			/* return code clear */
    }
    if(fflush(stderr) < 0) {		/* fflush return code check */
	rvl = 0;			/* return code clear */
    }
    return(rvl);			/* return code out */
 }
}
#else
#define ucrp_error(...) 
#endif							/* **/
int ucrp_debug(const char *frmt, ...) 	/* print debug message & exit **/
{
 if(ucrp_quiet == 1) {			/* no messages in quiet mode **/
    return(1);
 } else {
    char pbuf[1024];			/* message buffer **/
    int rvl;
    rvl = 1;
    va_list valt;				/* argument list **/
    va_start(valt, frmt);			/* init argument list **/
    u_int32_t cnt;
    cnt = vsnprintf(pbuf, sizeof(pbuf), frmt, valt);	/* **/
    if(cnt >= sizeof(pbuf)) {
	rvl = 0;
    }
    va_end(valt);				/* **/
    if(fprintf(stderr, UMSG_PS, pbuf) < 0) {/* print error message to stderr **/	
	rvl = 0;
    }
    if(fflush(stderr) < 0) {
	rvl = 0;
    }
    return(1);
 }
}
/* \\z - single paged printf, \\Z - multiple paged printf - \\E **/
#if 1
int ucrp_printf(const char *frmt, ... ) 
{
 static u_int8_t pmode = 0;		/* print mode **/
 static int prow = 0; 			/* screen row **/
 static int pcol = 0;			/* column number **/
 static char pbuf[4096];		/* print buffer **/
 u_int8_t nmode = 0;			/* next print mode **/
 va_list valt;				/* argument list **/
 if(ucrp_quiet == 1) {	
    return(1);
 }
 if((*frmt == '\\')) {			/* format **/
    if(*(frmt+1) == 'z') {
	nmode = 1;			/* single paged mode **/
	frmt += 2;			/* skip mode character */
    } else
    if(*(frmt+1) == 'Z') {
	nmode = 2;			/* multiple paged mode **/
	frmt += 2;			/* skip mode character */
    } else
    if(*(frmt+1) == 'E') {
	nmode = 0;			/* normal node **/
	frmt += 2;			/* skip mode character */
    }
 }
 int rvl;
 rvl = 1;
 if(nmode != pmode) {			/* change mode **/
    if(pmode == 0) {
	prow = 0; 			/* screen row **/
	pcol = 0;			/* column number **/
    }
    pmode = nmode;			/* save next mode **/
 } 
 va_start(valt, frmt);			/* init argument list **/
 u_int32_t cnt;
 cnt = vsnprintf(
    pbuf, 
    sizeof(pbuf), 
    frmt, 
    valt
 );
 if(cnt >= sizeof(pbuf)) {
    fprintf(				/* print in normat mode **/
	stderr, 
	"Out of printf buffer error, bytes {%d}\r\n",
	cnt
    );
    rvl = 0;
 }
 va_end(valt);
 if(pmode == 0) {			/* print in normat mode **/
    if(fprintf(stdout, UMSG_PS, pbuf) < 0) {	
	rvl = 0;
    }
    if(fflush(stdout) < 0) {
	rvl = 0;
    }
 } else 
 if(ucrp_quiet >= 2) {			/* quiet mode **/
    if(fprintf(stdout, UMSG_PS, pbuf) < 0) {
	rvl = 0;
    }
    if(fflush(stdout) < 0) {
	rvl = 0;
    }
 } else {
    int ppos;				/* buffer position **/
    ppos = 0;				/* start buffer pos **/
    while(ppos < (int) sizeof(pbuf)) {
	char pch;
	pch = *(pbuf + ppos);		/* read character **/
	if(pch == '\0') {		/* end of string **/
    	    break;
	}
	if(pch == '\r') {
	} else
	if(pch == '\n') {
    	    pcol = 0;			/* column number **/
    	    prow += 1;			/* **/
    	    if(fprintf(stdout, UMSG_CRNL) < 0) {	/* print character **/
		rvl = 0;
	    }
	} else
	if(pch == '\t') {
    	    int cnt;
    	    cnt = pcol;
    	    pcol = (((pcol + 8) / 8) * 8);
    	    cnt = pcol - cnt;
    	    while(cnt-- > 0) {
    		if(fprintf(stdout, UMSG_PC, ' ') < 0) {
		    rvl = 0;
		}
	    }
	} else {
	    pcol += 1;
	    if(fprintf(stdout, UMSG_PC, pch) < 0) {
		rvl = 0;
	    }
	}
	ppos++;
	if(pcol >= 80) {
    	    pcol -= 80;
    	    prow += 1;
	}
	if(prow >= 24) {		/* print cr lf **/
	    if(pch != '\n') {
    		if(fprintf(stdout, UMSG_CRNL) < 0) {
		    rvl = 0;
		}
	    }
    	    if(fprintf(stdout, UMSG_PRESS_ANY_KEY) < 0) {
		rvl = 0;
	    }
    	    if(fflush(stdout) < 0) {
		rvl = 0;
	    }
	    pch = '\0';			/* clear input buffer **/
	    if(ucrp_select(&pch, 1, 0, 1) < 0) { /* read character, no echo, wait **/
		pch = '\0';
    		if(fprintf(stderr, "Select error\r\n") < 0) { /* print cr lf **/
		    rvl = 0;
		}
	    }
    	    if(fprintf(stdout, UMSG_CRNL) < 0) { /* print cr lf **/
		rvl = 0;
	    }
    	    pcol = 0;			/* new column **/
	    prow = 0;			/* new row **/
	}
    }
    if(fflush(stdout) < 0) {		/* **/
	rvl = 0;
    }
    if(pmode == 1) {			
	pmode = 0;			/* switch from single page mode to normal **/
    }
 }
 return(rvl);
}
#else /* 0 **/
#define ucrp_printf printf
#endif							
/* test file existing and  mode **/
/* qual f - test regular file, l - test link, d - test directory,
r - test read, w - test write, x - test executable capability
o - other, g- group, u - user
**/
int ucrp_file(char *qual, char * frmt, ...)
{
 char * buf;				/* file name buffer **/
 buf = (char *) malloc(ucrp_path_size);	/* allocate buffer **/
 if(buf == 0) {				/* buffer alloc error **/
    return(0);
 }
 va_list valt;				/* variable argument list **/
 va_start(valt, frmt);			/* init variable argument list **/
 u_int32_t cnt; 			/* bytes in file name buffer */
 cnt = vsnprintf(			/* print file name to buffer with limited size **/
    buf, 
    ucrp_path_size, 
    frmt, 
    valt
 );
 if(cnt >= ucrp_path_size) {		/* out of buffer check */
    free(buf);				/* buffer free */
    buf = 0;				/* buffer pointer clear */
    return(-1);				/* file test error */
 }
 va_end(valt);				/* deinit variable argument list **/
 if(qual == 0) {			/* regular file **/
    int rvl;
    if(access(buf,F_OK) < 0) {		/* check file presence **/
        rvl = 0;			/* file not exist **/
    } else {
	rvl = 1;
    }
    free(buf);				/* free buffer allocated **/
    buf = 0;				/* clear buffer refference **/
    return(rvl);			/* file not exist **/
 }
 if((*(qual+0) == 'f') && (*(qual+1) == '\0')) {
    int rvl;
    if(access(buf,F_OK) < 0) {
        rvl = 0;			/* file not exist **/
    } else {
	rvl = 1;
    }
    free(buf);				/* free buffer allocated **/
    buf = 0;				/* clear buffer refference **/
    return(rvl);			/* file not exist **/
 }
 u_int8_t freg;				/* 0 - regular file, 1 - dir, 2 - link **/
 u_int8_t prot;				/* protection **/
 mode_t fmode;				/* file mode **/
 freg = 0;				/* regular file **/
 fmode = 0;				/* file mode (owner) **/
 prot = 0;				/* owner/user **/
 while(*(qual) != '\0') {
    switch(*qual) {	
	case 'f' : freg = 0; break;	/* regular file **/	
	case 'd' : freg = 1; break;	/* dirrectory **/
	case 'l' : freg = 2; break;	/* link **/
	case 'u' : prot = 0; break;	/* user/owner **/
	case 'g' : prot = 1; break;	/* group **/
	case 'o' : prot = 2; break;	/* other **/
	case 'r' : 
	         if(prot == 0) { fmode |= S_IRUSR; } /* owner can read **/
	    else if(prot == 1) { fmode |= S_IRGRP; } /* group can read **/
	    else if(prot == 2) { fmode |= S_IROTH; } /* other can read **/
	    break;	
	case 'w' : 
	         if(prot == 0) { fmode |= S_IWUSR; } /* user can write **/
	    else if(prot == 1) { fmode |= S_IWGRP; } /* group can write **/
	    else if(prot == 2) { fmode |= S_IWOTH; } /* other can write **/ 
	    break;
	case 'x' :
	         if(prot == 0) { fmode |= S_IXUSR; } /* user can execute **/
	    else if(prot == 1) { fmode |= S_IXGRP; } /* group can execute **/
	    else if(prot == 2) { fmode |= S_IXOTH; } /* other can execute **/
	    break;	
	default  : 				/* unknown qualificator **/
	    free(buf);				/* free buffer allocated **/
	    buf = 0;
	    return(0);				/* **/
    }
    qual += 1; 				/* move next character **/
 }
 struct stat stbuf;			/* file status **/
 if(stat(buf, &stbuf) < 0) {		/* read file status **/
    free(buf);				/* free buffer allocated **/
    buf = 0;				/* clear buffer refference **/
    return(0);
 }
 if(buf != 0) {
    free(buf);				/* free buffer allocated **/
    buf = 0;				/* clear buffer refference **/
 }
 if(freg == 0) {			/* regular file **/
    if(S_ISREG(stbuf.st_mode) == 0) {
	return(0);			/* not regular file **/
    }
 } else 
 if(freg == 1) {			/* dirrectory **/
    if(S_ISDIR(stbuf.st_mode) == 0) {
	return(0);			/* not directory **/
    }
 } else 
 if(freg == 2) {			/* file link **/
    if(S_ISLNK(stbuf.st_mode) == 0) {
	return(0);			/* not file link **/
    }
 } else {
    return(0);
 }
 if((stbuf.st_mode & fmode) == fmode) {
    return(1);				/* wrx mode **/
 } else {
    return(0);
 }
}
int ucrp_unlink(char * frmt, ...)	/* unlink file **/
{
 char * buf;				/* file name buffer **/
 buf = (char *)malloc(ucrp_path_size);	/* buffer allocation **/
 if(buf == 0) {				
    return(0);				/* buffer allocation **/
 }
 va_list valt;				/* variable argument list **/
 va_start(valt, frmt);			/* init variable argument list **/
 u_int32_t cnt;				/* bytes in buffer */
 cnt = vsnprintf(			/* print file name to buffer with limited size **/
    buf, 
    ucrp_path_size,
    frmt,
    valt
 );
 va_end(valt);				/* deinit variable argument list **/
 if(cnt >= ucrp_path_size) {		/* out of buffer check */
    free(buf);				/* file buffer free */
    buf = 0;				/* file name buffer pointer clear */
    return(-1);
 } 
 int rvl;
 if(unlink(buf) < 0) {			/* unlink file **/
    rvl = 0;				/* unlink file error **/
 } else {
    rvl = 1;
 }
 if(buf != 0) {
    free(buf);				/* free buffer allocated **/
    buf = 0;				/* clear buffer refference **/
 }
 return(rvl);
}
int ucrp_clear(char * frmt, ...)	/* clear file **/
{
 char * buf;				/* file name buffer **/
 buf = (char *)malloc(ucrp_path_size);	/* buffer allocation **/
 if(buf == 0) {				
    return(0);				/* buffer allocation **/
 }
 int rvl;				/* return code **/
 rvl = 1;				/* start return value **/
 va_list valt;				/* argument list **/
 va_start(valt, frmt);			/* init argument list **/
 u_int32_t cnt;				/* bytes in buffer */
 cnt = vsnprintf(			/* form file name */
    buf,				/* file name buffer */
    ucrp_path_size,			/* file name buffer size */
    frmt,				/* format */
    valt				/* argument list */
 );
 va_end(valt);
 if(cnt >= ucrp_path_size) {		/* buffer out of check */
    free(buf);				/* file name buffer free */
    buf = 0;				/* file name buffer pointer clear */
    return(-1);
 }
 FILE * file;				/* file **/
 file = fopen(buf, "r+b");		/* open file **/
 if(buf != 0) {				/* file buffer check */
    free(buf);				/* file buffer free */
    buf = 0;				/* file buffer pointer clear */
 }
 if(file == 0) {	/* file open error -> print error message & exit **/	
    ucrp_error("File {%s} open error\r\n", 
	buf
    );
    return(0);
 }
 if(fseek(file, 0, SEEK_END) < 0) {	/* seek file end **/
    rvl = 0;				/* clear return code **/
 }
 int len;				/* file length **/
 len = ftell(file);			/* read file length **/
 if(fseek(file, 0, SEEK_SET) < 0) {	/* rewind file **/
    rvl = 0;				/* seeking error -> clear return code **/
 }
 if(ucrp_clear_mode == UCRP_CLEAR_INC) { /* incrtement mode clear test */
    u_int8_t wch;			/* character writed */
    wch = 0;				/* character  writed start value */
    int pos;				/* file position **/
    for(pos = 0; pos < len; pos++) {
	if(fwrite(&wch, 1, 1, file) < 0) {	/* write word **/
	    rvl = 0;
	}
    }
 } else {
    u_int32_t wch;			/* character writed */
    if(ucrp_clear_mode == UCRP_CLEAR_CLR) {/* null mode clear test */
	wch = 0;			/* character writed start value */
    } else
    if(ucrp_clear_mode == UCRP_CLEAR_SET) {/* set mode clear test */
	wch = (~0);			/* character writed start value */
    } else {
	wch = 0;
    }
    int pos;				/* file position **/
    for(pos = 0; pos < len;) {
	pos += sizeof(pos);		/* next file position **/
	if(pos < len) {
	    fwrite(&wch, 4, 1, file);	/* write word **/
	} else {
	    fwrite(&wch, (len & 3), 1, file); /* write remainimg bytes  **/
	}
    }
 }
 fclose(file);				/* close file **/
 return(rvl);				/* return return value **/ 
}






int ucrp_usage(int scmd)		/* print help message **/
{
 if(ucrp_quiet == 1) {			/* **/
    ucrp_quiet = 2;
 }
    ucrp_printf(
	"\\z"
	UCRP_USAGE
	"Usage : ucrp [options] <input file> [<output file>]\r\n"
	"where options may be:\r\n"
	"--license      - show this program license agreement\r\n"
	"--license-ru   - show this program license agreement translated for russian language\r\n"
	"--encrypt      - encrypt file\r\n"
	"--decrypt      - decrypt file\r\n"
	"--passwd <passwd> | request - password\r\n"
	"--hide-passwd [*|$|d] - hide passwd\r\n"
	"--show-passwd  - show passwd\r\n"
	"--block-size   - block size\r\n"
	"--change-dir   - change current directory\r\n"
	"--info         - print encrypt file info message and exit\r\n"
	"--clear [inc|0,1] - clear current directory disk space and bash & mc history\r\n"
	"                 inc write increment byte, 0 - write zero, 1 - write FF\r\n"
	"--disable "
#ifdef  __UCRP_XOR_ENABLE__
	"xor|"				/* disable xor **/
#endif							/* __UCRP_XOR_ENABLE__ **/
#ifdef __UCRP_ADD_ENABLE__
	"add|"				/* disable add **/
#endif							/* __UCRP_ADD_ENABLE__ **/
#ifdef __UCRP_ROT_ENABLE__
	"rot|"				/* disable rotate **/
#endif							/* __UCRP_ROT_ENABLE__ **/
#ifdef __UCRP_TAB_ENABLE__
	"tab|"				/* disable table **/
#endif							/* __UCRP_TAB_ENABLE__ **/
#ifdef __UCRP_ECC_ENABLE__
	"ecc|"				/* disable ecc **/
#endif							/* __UCRP_ECC_ENABLE__ **/
#ifdef __UCRP_BTS_ENABLE__
	"bts|"				/* disable bts **/
#endif 							/* __UCRP_BTS_ENABLE__ **/
#ifdef __UCRP_BEV_ENABLE__
	"bev|"				/* disable bev **/
#endif 							/* __UCRP_BEV_ENABLE__ **/
#ifdef __UCRP_XRE_ENABLE__
	"xre|"				/* disable xre **/
#endif							/* __UCRP_XRE_ENABLE__ **/
	"std - disable algoritms\r\n"
	"--enable "
#ifdef  __UCRP_XOR_ENABLE__
	"xor|"				/* enable xor **/
#endif							/* __UCRP_XOR_ENABLE__ **/
#ifdef __UCRP_ADD_ENABLE__
	"add|"				/* enable add **/
#endif							/* __UCRP_ADD_ENABLE__ **/
#ifdef __UCRP_ROT_ENABLE__
	"rot|"				/* enable rotate **/
#endif							/* __UCRP_ROT_ENABLE__ **/
#ifdef __UCRP_TAB_ENABLE__
	"tab|"				/* enable table **/
#endif							/* __UCRP_TAB_ENABLE__ **/
#ifdef __UCRP_ECC_ENABLE__
	"ecc|"				/* enable ecc **/
#endif							/* __UCRP_ECC_ENABLE__ **/
#ifdef __UCRP_BTS_ENABLE__
	"bts|"				/* enable bts **/
#endif 							/* __UCRP_BTS_ENABLE__ **/
#ifdef __UCRP_BEV_ENABLE__
	"bev|"				/* enable bev **/
#endif 							/* __UCRP_BEV_ENABLE__ **/
#ifdef __UCRP_XRE_ENABLE__
	"xre|"				/* enable xre **/
#endif							/* __UCRP_XRE_ENABLE__ **/
	"std - enable algoritms\r\n"
	"--quiet        - quiet mode\r\n"
	"--version      - print version and exit\r\n"
	"--todo         - print todo and exit\r\n"
	"--changelog    - print changelog and exit\r\n"
	"--man          - print man page and exit\r\n"
	"--help         - print help note and exit\r\n",
	ucrp_version_hi,		/* version hight number **/
	ucrp_version_lo			/* version low **/
    );
 return(1);
}
int ucrp_man(int scmd)			/* **/
{
 ucrp_printf(
    "\\zMAN page ucrp %d.%d\r\n"
    "Usage : ucrp [options] <input file> [<output file>]\r\n"
    "--license      - show this program license agreement\r\n"
    "--license-ru   - show this program license agreement translated for russian language\r\n"
    "--encrypt      - encrypt file, example:\r\n"
    "    ucrp --encrypt <input file> <output file> - encrypt file with empty passwd\r\n"
    "--decrypt      - decrypt file\r\n"
    "passwd options:\r\n"
    "--passwd <passwd> | request - password from command line or keyboard\r\n"
    "--hide-passwd [*|$|d] - hide passwd, print characters * or # insted typed chars\r\n"
    "\tor d - print nofing insted typed characters, default - print all input\r\n"
    "--show-passwd  - show passwd\r\n"
    "--block-size   - block size\r\n"
    "--change-dir   - change current directory\r\n"
    "--info         - print encrypt file info message and exit\r\n"
    "--clear [inc|0,1] - clear current directory disk space and bash & mc history\r\n"
    "                 inc write increment byte, 0 - write zero, 1 - write FF\r\n"
    "--disable "
#ifdef  __UCRP_XOR_ENABLE__
    "xor|"				/* disable xor **/
#endif							/* __UCRP_XOR_ENABLE__ **/
#ifdef __UCRP_ADD_ENABLE__
    "add|"				/* disable add **/
#endif							/* __UCRP_ADD_ENABLE__ **/
#ifdef __UCRP_ROT_ENABLE__
    "rot|"				/* disable rotate **/
#endif							/* __UCRP_ROT_ENABLE__ **/
#ifdef __UCRP_TAB_ENABLE__
    "tab|"				/* disable table **/
#endif							/* __UCRP_TAB_ENABLE__ **/
#ifdef __UCRP_ECC_ENABLE__
    "ecc|"				/* disable ecc **/
#endif							/* __UCRP_ECC_ENABLE__ **/
#ifdef __UCRP_BTS_ENABLE__
    "bts|"				/* disable bts **/
#endif 							/* __UCRP_BTS_ENABLE__ **/
#ifdef __UCRP_BEV_ENABLE__
    "bev|"				/* disable bev **/
#endif 							/* __UCRP_BEV_ENABLE__ **/
#ifdef __UCRP_XRE_ENABLE__
    "xre|"				/* disable xre **/
#endif							/* __UCRP_XRE_ENABLE__ **/
    "std - disable algoritms\r\n"
    "--enable "
#ifdef  __UCRP_XOR_ENABLE__
    "xor|"				/* enable xor **/
#endif							/* __UCRP_XOR_ENABLE__ **/
#ifdef __UCRP_ADD_ENABLE__
    "add|"				/* enable add **/
#endif							/* __UCRP_ADD_ENABLE__ **/
#ifdef __UCRP_ROT_ENABLE__
    "rot|"				/* enable rotate **/
#endif							/* __UCRP_ROT_ENABLE__ **/
#ifdef __UCRP_TAB_ENABLE__
    "tab|"				/* enable table **/
#endif							/* __UCRP_TAB_ENABLE__ **/
#ifdef __UCRP_ECC_ENABLE__
    "ecc|"				/* enable ecc **/
#endif							/* __UCRP_ECC_ENABLE__ **/
#ifdef __UCRP_BTS_ENABLE__
    "bts|"				/* enable bts **/
#endif 							/* __UCRP_BTS_ENABLE__ **/
#ifdef __UCRP_BEV_ENABLE__
    "bev|"				/* enable bev **/
#endif 							/* __UCRP_BEV_ENABLE__ **/
#ifdef __UCRP_XRE_ENABLE__
    "xre|"				/* enable xre **/
#endif							/* __UCRP_XRE_ENABLE__ **/
    "std - enable algoritms\r\n"
    "--quiet        - quiet mode\r\n"
    "--version      - print version and exit\r\n"
    "--todo         - print todo and exit\r\n"
    "--changelog    - print changelog and exit\r\n"
    "--man          - print man page and exit\r\n"
    "--help         - print help note and exit\r\n"
    ,ucrp_version_hi,			/* version hight number **/
    ucrp_version_lo			/* version low **/
 );
 return(1);
}
int ucrp_license(void)	/* cutted version - if you have source codes */
{ /* of this software, you must provide this ability to another users **/
 ucrp_printf(UCRP_LICENSE,
    ucrp_version_hi,			/* version hight number **/
    ucrp_version_lo			/* version low **/
 );
 return(1);
}
int ucrp_license_ru(void) /*   -      */
{ /*   ,      */
 ucrp_printf(UCRP_LICENSE_RU,	/*    **/
    ucrp_version_hi,			/* version hight number **/
    ucrp_version_lo			/* version low **/
 );
 return(1);
}
int ucrp_version(void)			/* print version **/
{
 ucrp_printf(UMSG_VERSION, 
    ucrp_version_hi,			/* version hight number **/
    ucrp_version_lo			/* version low **/
 );
 return(1);
}
int ucrp_todo(void)			/* print todo note **/
{
 char * msg = 
    "";
/* **/
 if(ucrp_quiet == 1) {
    ucrp_quiet = 2;
 }
 ucrp_printf(msg);
 return(1);
}
int ucrp_changelog(void)			/* print changelog note **/
{
 if(ucrp_quiet == 1) {
    ucrp_quiet = 2;
 }
 ucrp_printf(
    UMSG_CHANGELOG
 );
 return(1);
}

int ucrp_init(void)
{
 int pos;
 for(pos = 0; pos < (int) sizeof(ucrp_passwd); ++pos) {
    ucrp_passwd[pos] = 0;
 }
 return(1);
}
int ucrp_key(u_int32_t key, u_char * buf, int len) /* read key, return key length bytes */
{
 if(buf == 0 || len == 0) {
    return(0);
 }
 if(key <= 0xFF) {			/* key length - one byte **/
    u_int8_t bchr;			/* buffer character **/
    bchr = *(buf + 0);			/* read characeter from buffer **/
    if(bchr == key) {			
        return(1);
    } else {
        return(0);			/* one byte key string not found in buffer **/
    }
 } else 
 if(key <= 0xFFFF) {			/* two bytes character **/
    if(len < 2) {			/* **/
	return(0);
    } else {
        u_int16_t bchr;			/* two bytes character **/
	bchr = 				/* calculate two bytes character **/
	    ( (((u_int32_t) *(buf + 1)) << 8) | 
	       ((u_int32_t) *(buf + 0)) );
	if(key == bchr) {		/* two bytes character found in buffer **/
	    return(2);
	} else {
	    return(0);
	} 
    }
 } else 
 if(key <= 0xFFFFFF) {			/* free bytes character **/
    if(len < 3) {
	return(0);
    } else {
        u_int32_t bchr;
	bchr = ( (((u_int32_t) *(buf + 2)) << 16) | 
	    (((u_int32_t) *(buf + 1)) << 8) | ((u_int32_t) *(buf + 0)) );
	if(key == bchr) {
	    return(3);
	} else {
	    return(0);
	}
    }
 } else {				/* for bytes character **/
    if(len < 4) {
	return(0);
    } else {
	u_int32_t bchr;
	bchr = 
	( (((u_int32_t) *(buf + 3)) << 24) | 
	  (((u_int32_t) *(buf + 2)) << 16) | 
	  (((u_int32_t) *(buf + 1)) << 8) | 
	   ((u_int32_t) *(buf + 0)) );
	if(key == bchr) {
	    return(4);
	} else {
	    return(0);
	}
    }
 }
}
/* echo = 0 - no echo, 1 - echo, 2 - '*' echo, 3 - '$' echo**/ 
/* wait - 0 = no wait, 1 = wait **/
int ucrp_select(char * buf, int len, int echo, int wait)
{
 fd_set r_fds; 				/* read mask **/
 FD_ZERO(&r_fds);			/* clear read mask **/
/* read stdin file descriptor **/
 int stdinfd;				/* standart input file descriptor **/
 stdinfd = fileno(stdin);		/* read standart input file descriptor **/
 if((stdinfd < 0) || (stdinfd >= (int)sizeof(r_fds) * 8)) {
    ucrp_error(UMSG_STDIN_ERROR);	/* print error message **/
    return(-1);				/* error **/
 }
 int maxfd;				/* maximum descriptor number **/
 maxfd = stdinfd;			/* set maximal descriptor number **/
#define TC_GET_SET_ATTR
#ifdef TC_GET_SET_ATTR
 struct termios stdintios;		/* standart input attributes **/
 if(tcgetattr(stdinfd, &stdintios) < 0) { /* read attributes **/
    ucrp_error(UMSG_TCGETATTR_STDIN_ERROR, errno);
    return(-1);
 }
 struct termios tios;			/* modified attributes **/
 tios = stdintios;			/* copy attributes **/
 cfmakeraw(&tios);			/* raw stdin **/
 if(
    tcsetattr(
	stdinfd, 			/* standart input file descriptor **/
	TCSANOW, 			/* set immedeatly **/
	&tios				/* raw atrtributes **/
    ) < 0) { 
    ucrp_error(UMSG_TCSETATTR_STDIN_ERROR, errno);
    return(-1);
 }        
#endif 
 int stdinval;				/* stdin settings **/
 stdinval = fcntl(stdinfd ,F_GETFL,0);	/* read stdin settings **/
 if(fcntl(stdinfd, F_SETFL, O_NONBLOCK) < 0) {	
    ucrp_error(				/* set non block mode error **/
	"Set nonblock mode for standart input error\n");
    return(-1);				/* error **/
 }
 int pos;				/* destination buffer position **/
 int rvl;				/* return code **/
 pos = 0;				/* start buffer position **/
 while(pos < len) {			
    u_int8_t bch;			/* buffer character **/
    bch = *(buf + pos);			/* read buffer character **/
    if(bch == '\0') {			/* string separator **/
	break;
    }
    if(echo == 1) {
	ucrp_printf(UMSG_PC,bch);	/* print character **/
    } else 
    if(echo == 2) {
        ucrp_printf(UMSG_PC,'*');
    } else
    if(echo == 3) {
        ucrp_printf(UMSG_PC,'$');
    }
    pos += 1;				/* next buffer position **/
 }
 rvl = 1; 				/* start return value **/
 while(1) {
    FD_SET(
	stdinfd	,			/* stdin descriptor **/
	& r_fds				/* read descriptor **/
    );     
    struct timeval tval;		/* time interval **/
    if(wait > 0) {
        tval.tv_sec = wait;		/* 1sec delay **/
    } else {
        tval.tv_sec = 0;		/* no wait **/
    }
    tval.tv_usec = 0;			/* usec **/
    int cnt;				/* io events **/
    cnt = select(
        maxfd + 1,			/* descriptors in mask **/
	& r_fds, 			/* read mask **/
	0, 				/* write mask **/
	0, 				/* exeption mask **/
	& tval				/* time interval **/
    );
    if(kill_link > 1) {			/* received more then one term signal **/
	{ break; }			/* -> exit from loop **/
    }
    if(cnt < 0) {			/* select error **/
        if(
    	    (errno == EWOULDBLOCK) ||  /* opeation blocked **/
	    (errno == EINTR)
	) {	
	    continue;
	} else {
    	    rvl = -1;  		/* clear return code **/
	    ucrp_error(UMSG_SELECT_ERROR,  /* print error message **/
	        errno
	    );
    	    { break; }
	}
    } else
    if(cnt == 0) {			/* no events **/
        if(wait == 0) {
    	    { break; }			/* no wait -> exit from loop **/
	} else {
    	    continue;
	}
    }
    if(FD_ISSET(stdinfd,& r_fds) == 0) {
	continue;			/* not set standart input descriptor **/
    }
    u_char rbuf[32];			/* read buffer **/
    rbuf[0] = 0;
    while(1) {
	cnt = read(stdinfd, rbuf, sizeof(rbuf));
	if(cnt >= 0) {
	    { break; }
	}  
	if(
	    (errno == EWOULDBLOCK) || 	/* opeation blocked **/
	    (errno == EINTR)		/* operation iterrupted **/
	) {	
	    continue;
	} else {
    	    rvl = -1;  			/* read error**/
    	    ucrp_error(UMSG_STDIN_READ_ERROR, /* print error message **/
	        errno
	    );
    	    { break; }
	}
    }
    if(rvl == -1) {
	{ break; }
    }
    if(kill_link > 1) {			/* receive more then one **/
	{ break; }			/* -> exit from loop **/
    } 
    if(cnt <= 0) {			/* empty buffer **/
        continue;			/* -> next read **/
    }
    int rpos = 0;			/* read postion **/
    while(rpos < cnt) {
        u_char pkey;			/* del or backspace key **/
        u_char plen;			/* key length **/
	plen = ucrp_key(UCRP_BS, rbuf, cnt);	/* check BS **/
	if(plen > 0) { 
	    pkey = 1;			/* BS key **/
	} else {
	    plen = ucrp_key(UCRP_DEL, rbuf, cnt); /* check DEL **/	
	    if(plen > 0) { 
	        pkey = 1;		/* DEL key **/
	    } else {
	        pkey = 0;		/* not DEL and not BS keys **/
	    }
	}
	if(pkey == 1) {
	    rpos += plen;		/* next buffer position **/
	    if(pos > 0) {
	        pos -= 1;		/* previous character position **/
	        if(buf != 0) {		/* not null buffer **/
	            if(pos < len) {	
	    	        *(buf + pos) = '\0'; /* write terminating character **/
		    }
		}
		if(
		    (echo == 1) || 	/* print character as is **/
		    (echo == 2) || 	/* print * as typed character **/
		    (echo == 3)		/* print # as typed character **/
		) {
		    ucrp_printf(UMSG_BS);
	    	    ucrp_printf(UMSG_SP);
		    ucrp_printf(UMSG_BS);
		}
	    }
	} else {
	    u_char rch;			/* readed character **/
	    rch = rbuf[rpos];		/* read character from buffer **/
	    if((rch == '\n') || (rch == '\r')) {
	        ucrp_printf(UMSG_CRNL);	/* enter **/
	        { break; }		/* string entered **/
	    } else
	    if((rch == 0x1B)) {		/* escape key **/
	        pos = 0;
	        ucrp_printf("{Canceled}");
	        { break; }		/* string cancelled **/
	    } else {		/* valid character -> write character to buffer **/	
	        if(buf != 0) {		/* not null pointer **/
	    	    if(pos < len) {	/* position in buffer **/
		        if(pos >= 0) {	
			    *(buf + pos) = rch;
			}
		    }
		}
	        ++pos;			/* next buffer position **/
	        if(echo == 1) {
	    	    ucrp_printf(UMSG_PC,rch);
		} else 
		if(echo == 2) {
		    ucrp_printf(UMSG_PC,'*');
		} else
		if(echo == 3) {
		    ucrp_printf(UMSG_PC,'$');
		}
	    }
	    rpos += 1;    		/* next readed character **/
	} 
    }
    if(wait == 0) {			/* no wait mode **/
        { break; }			/* clear more **/
    }
    if(rpos < cnt) {			
        { break; }			/* read character processing terminating **/
    }
 }
 if(buf != 0) {				/* write string teminator **/
    if(pos > len) {
	pos = len - 1;
    }
    if(pos > 0) {
        *(buf + pos) = '\0';
    }
 }
 if(fcntl(stdinfd , F_SETFL, stdinval) < 0) {	
    ucrp_error(				/* restore standart input **/
	"Restore setting for standart input error\n");
    return(-1);				/* error **/
 }
#ifdef TC_GET_SET_ATTR
 if(tcsetattr(stdinfd, TCSANOW, &stdintios) < 0) {
    ucrp_error(UMSG_TCSETATTR_STDIN_ERROR, errno);
    rvl = -1;
 }        
#endif 
 if(rvl < 0) {
    return(rvl);
 } else {
    return(pos);
 }
}
/* init random **/
int ucrp_random_fd = -1;		/* /dev/random file descriptor **/
int ucrp_urandom_fd = -1;		/* /dev/urandom file descriptor **/
int ucrp_random_init(void)
{
 int rvl;				/* return code */
 rvl = 1;				/* start return code */
 ucrp_random_fd = open(			/* open random in nonblock mode **/
    UMSG_RANDOM_FILE_NAME,		/* /dev/random */
    (O_RDONLY | O_NONBLOCK)		/* read only nonblock mode */
 );
 if(ucrp_random_fd >= 0) {		/* /dev/random open check */
    int fdval;
    fdval = fcntl(ucrp_random_fd, F_GETFL);	
    if(fdval < 0) {			/* read error check */
	ucrp_error(			/* error message print */
	    UMSG_GET_FCNTL_ERROR,	/* format */
	    UMSG_RANDOM_FILE_NAME	/* file name /dev/random */
	);
	rvl = 0;			/* return code clear */
    }
    if(fcntl(ucrp_random_fd, F_SETFL, (fdval | O_NONBLOCK)) < 0) {
	ucrp_error(UMSG_O_NONBLOCK_ERROR);
	rvl = 0;
    }
 }
 ucrp_urandom_fd = open(		/* open urandom in nonblock mode **/
    UMSG_URANDOM_FILE_NAME,		/* /dev/urandom */
    (O_RDONLY | O_NONBLOCK)		/* read only nonblock mode */
 );
 if(ucrp_urandom_fd >= 0) {
    int fdval;
    fdval = fcntl(ucrp_urandom_fd, F_GETFL);	
    if(fdval < 0) {
	ucrp_error(
	    UMSG_GET_FCNTL_ERROR,
	    UMSG_URANDOM_FILE_NAME
	);
	rvl = 0;
    }
    if(fcntl(ucrp_urandom_fd, F_SETFL, (fdval | O_NONBLOCK)) < 0) {
	ucrp_error(UMSG_O_NONBLOCK_ERROR);
	rvl = 0;
    }
 }
 if((ucrp_random_fd >= 0) || (ucrp_urandom_fd >= 0)) {
    return(rvl);
 } else {
    return(0);
 }
}
int ucrp_random_close(void)		/* close random files **/
{
 if(ucrp_random_fd != -1) {
    close(ucrp_random_fd);		/* close randowm file **/
    ucrp_random_fd = -1;		
 }
 if(ucrp_urandom_fd != -1) {
    close(ucrp_urandom_fd);		/* close urandowm file **/
    ucrp_urandom_fd = -1;
 }
 return(1);
}
u_char ucrp_random(void)		/* read randowm byte **/
{
 static u_int32_t rch = 0;
 if(ucrp_random_fd != -1) {
    if(read(ucrp_random_fd, &rch, 1) == 1) {
	return(rch);
    } 
 }
 if(ucrp_urandom_fd != -1) {
    if(read(ucrp_urandom_fd, &rch, 1) == 1) {
	return(rch);
    } 
 }
 rch = (rch << 11) | (rch >> 21); 
 rch += 1;
 return((u_char)rch); 
}
int ucrp_comp(int pos, int val) 	/* read/write compatibility bit */
{
 u_int32_t byte;			/* byte number */
 u_int32_t bit;				/* bit number */
 byte = (pos >> 3);			/* byte **/
 bit = (pos & 0x7);			/* bit **/
 if(byte >= sizeof(ucrp_comp_buf)) {
    return(0);
 }
 if(val < 0) {				/* read bit value **/
    return(((ucrp_comp_buf[byte] >> (7-bit)) & 0x1));
 } else {
    if(val) {
	ucrp_comp_buf[byte] |=  (0x80 >> bit);
    } else {
	ucrp_comp_buf[byte] &= ~(0x80 >> bit);
    }
    return(val);
 }
}
int ucrp_comp_val(int pos, int len, int val)	/* read/write compatibility value **/
{
 if(val < 0) {				/* read **/
    val = 0;
    while(len > 0) {
	val <<= 1;			/* left shift **/
	val |= ucrp_comp(pos, -1);	/* read bit **/
	pos += 1;			/* next bit number **/
	len  -= 1;			/* remain bits **/
    }
    return(val);
 } else {
    while(len > 0) {
	if((val & (1 << (len - 1))) != 0) {
	    ucrp_comp(pos,1);
	} else {
	    ucrp_comp(pos,0);
	}
	pos += 1;			/* next bit number **/
	len -= 1;			/* bits remain **/
    }	
    return(val);
 }
}
int ucrp_comp_init(void)		/* init compatibylity buffer **/
{
 int cpos;				/* clear compatibility buffer **/
 for(cpos = 0; cpos < (int)sizeof(ucrp_comp_buf); ++cpos) {
    ucrp_comp_buf[cpos] = 0;
 }
 ucrp_comp_buf[0] = 'u';		/* program name **/	
 ucrp_comp_buf[1] = 'c';		/* uenc 1.0 **/
 ucrp_comp_buf[2] = 'r';
 ucrp_comp_buf[3] = 'p';
 ucrp_comp_buf[4] = '1';
 ucrp_comp_buf[5] = '.';
 ucrp_comp_buf[6] = '1';
 ucrp_comp_buf[7] = '8';
#ifdef __UCRP_XOR_ENABLE__
 if((ucrp_enable & UCRP_XOR_ENABLE) != 0) { /* enable **/
    ucrp_comp(ucrp_xor_comp, 1);	/* xor compatibility **/
 }
#endif 					/* __UCRP_XOR_ENABLE__ **/
#ifdef __UCRP_ADD_ENABLE__
 if((ucrp_enable & UCRP_ADD_ENABLE) != 0) { 
    ucrp_comp(ucrp_add_comp, 1);	/* enable add compatibility **/
 }
#endif 					/* __UCRP_ADD_ENABLE__ **/
#ifdef __UCRP_ROT_ENABLE__
 if((ucrp_enable & UCRP_ROT_ENABLE) != 0) {
    ucrp_comp(ucrp_rot_comp, 1);	/* enable byte rotate **/
 }
#endif 					/* __UCRP_ROT_ENABLE__ **/
#ifdef __UCRP_TAB_ENABLE__
 if((ucrp_enable & UCRP_TAB_ENABLE) != 0) {
    ucrp_comp(ucrp_tab_comp, 1);	/* enable byte table **/
 }
#endif 					/* __UCRP_TAB_ENABLE__ **/
#ifdef __UCRP_ECC_ENABLE__
 if((ucrp_enable & UCRP_ECC_ENABLE) != 0) {
    ucrp_comp(ucrp_ecc_comp, 1);	/* enable ecc coding **/	
 }
#endif 					/* __UCRP_ECC_ENABLE__ **/
#ifdef __UCRP_BTS_ENABLE__
 if((ucrp_enable & UCRP_BTS_ENABLE) != 0) {
    ucrp_comp(ucrp_bts_comp, 1);	/* enable enable BTS {byte swapping} **/	
 }
#endif 					/* __UCRP_BTS_ENABLE__ **/
#ifdef __UCRP_XRE_ENABLE__
 if((ucrp_enable & UCRP_XRE_ENABLE) != 0) {
    ucrp_comp(ucrp_xre_comp, 1);		/* enable xor extended compatibility **/
 } 
#endif 					/* __UCRP_XRE_ENABLE__ **/
 ucrp_comp_val(		/* write source buffer size to compatibility buffer **/
    UCRP_SBL, UCRP_SBL_LEN, ucrp_src_buf_size); 
 ucrp_comp_val(		/* write destination buffer size to compatibility buffer **/
    UCRP_DBL, UCRP_DBL_LEN, ucrp_dst_buf_size);
 return(1);
}
int ucrp_comp_check(void)		/* check compatibility buffer **/
{
 if(
    (
	(ucrp_comp_buf[0] == 'u') &&	/* uenc test */
	(ucrp_comp_buf[1] == 'e') &&
	(ucrp_comp_buf[2] == 'n') &&
	(ucrp_comp_buf[3] == 'c')
    ) || 
    (
	(ucrp_comp_buf[0] == 'u') &&	/* ucrp test */
	(ucrp_comp_buf[1] == 'c') &&
	(ucrp_comp_buf[2] == 'r') &&
	(ucrp_comp_buf[3] == 'p')
    ) 
 ) {
    if(
	(ucrp_comp_buf[4] == ' ') &&	/* uenc 1.0 test */
        (ucrp_comp_buf[5] == '1') &&
	(ucrp_comp_buf[6] == '.') &&
        (ucrp_comp_buf[7] == '0')
    ) {
	return(1);				/* uenc 1.0 file **/
    } else 
    if(
	(ucrp_comp_buf[4] == '1') &&
	(ucrp_comp_buf[5] == '.') &&
	(
	    ucrp_comp_buf[6] >= '0' && 
	    ucrp_comp_buf[6] <= '9'
	) &&
	(
	    ucrp_comp_buf[7] >= '0' && 
	    ucrp_comp_buf[7] <= '9'
	)
    ) {
        return(1);
    } else {
        return(0);
    }
 } else {
    return(0);				/* not valid file */
 }
}
u_int32_t ucrp_comp_version(void)	/* return file version */
{
 if(
    (
	(ucrp_comp_buf[0] == 'u') &&	/* uenc test */
	(ucrp_comp_buf[1] == 'e') &&
	(ucrp_comp_buf[2] == 'n') &&
	(ucrp_comp_buf[3] == 'c')
    ) || 
    (
	(ucrp_comp_buf[0] == 'u') &&	/* ucrp test */
	(ucrp_comp_buf[1] == 'c') &&
	(ucrp_comp_buf[2] == 'r') &&
	(ucrp_comp_buf[3] == 'p')
    ) 
 ) {
    if(
	(ucrp_comp_buf[4] == ' ') &&	/* uenc 1.0 test */
        (ucrp_comp_buf[5] == '1') &&
	(ucrp_comp_buf[6] == '.') &&
        (ucrp_comp_buf[7] == '0')
    ) {
	return(100);			/* return version 100 */
    } else 
    if(
	(ucrp_comp_buf[4] == '1') &&
	(ucrp_comp_buf[5] == '.') &&
	(
	    ucrp_comp_buf[6] >= '0' && 
	    ucrp_comp_buf[6] <= '9'
	) &&
	(
	    ucrp_comp_buf[7] >= '0' && 
	    ucrp_comp_buf[7] <= '9'
	)
    ) {
	u_int32_t uver;			/* ucrp file version */
	uver = 100 + 			/* version hight number */			
	    (ucrp_comp_buf[6] - '0') * 10 +  /* version lo number hi digit */
	    (ucrp_comp_buf[7] - '0');	/* version lo number lo digit */
        return(uver);
    } else {
        return(0);			/* not ucrp file, version 0 */
    }
 } else {
    return(0);				/* not ucrp file, version 0 */
 }
}
int ucrp_key(int pos,int val)		/* read bit from key buffer **/
{					/* **/
 u_int32_t byte;			/* byte number */	
 u_int32_t bit;				/* bit number */	
 byte = (pos >> 3);			/* byte **/
 bit = (pos & 0x7);			/* bit **/
 if(byte >= sizeof(ucrp_key_buf)) {	/* **/
    return(0);				/* **/
 }					/* **/
 if(val < 0) {				/* read bit value **/
    return(((ucrp_key_buf[byte] >> (7-bit)) & 0x1));
 } else {				/* **/	
    if(val) {				/* **/
	ucrp_key_buf[byte] |=  (0x80 >> bit); /* **/
    } else {				/* **/
	ucrp_key_buf[byte] &= ~(0x80 >> bit); /* **/
    }					/* **/
    return(val);			/* **/
 }
}
int ucrp_key_val(int pos,int len, int val) /* read value from key buffer **/	
{
 if(val < 0) {
    val = 0;
    while(len > 0) {
	val <<= 1;
	val |= ucrp_key(pos, -1);
	pos += 1;
	len -= 1;
    }
    return(val);
 } else {
    while(len > 0) {
	if((val & (1 << (len - 1))) != 0) {
	    ucrp_key(pos,1);
	} else {
	    ucrp_key(pos,0);
	}
	pos += 1;
	len  -= 1;
    }	
    return(val);
 }
}
int ucrp_key_init(void)			/* init key buffer **/
{
 u_int32_t pos;
 for(pos = 0; pos < sizeof(ucrp_key_buf); ++pos) {
    ucrp_key_buf[pos] = 0;
 }
/* byte algs enable **/
#if defined __UCRP_XOR_ENABLE__ || defined __UCRP_XRE_ENABLE__
/* defined xor or xre algorithm **/
 u_char xor_val;			/* xor **/
 xor_val =  ucrp_random();		/* xor value **/
 ucrp_key_val(UCRP_XOR_BASE,UCRP_XOR_LEN,xor_val); 	/* xor value **/	
#endif 							/* __UCRP_XOR_ENABLE__ **/
#ifdef __UCRP_ADD_ENABLE__
 u_char add_val;			/* add **/
 add_val =  ucrp_random();		/* **/
 ucrp_key_val(UCRP_ADD_BASE,UCRP_ADD_LEN,add_val);	/* add value **/
#endif 							/* __UCRP_ADD_ENABLE__ **/
#ifdef __UCRP_ROT_ENABLE__					
 u_char rot_val;			/* rot **/
 rot_val = (ucrp_random() & 0x7);	/* **/
 ucrp_key_val(UCRP_ROT_BASE,UCRP_ROT_LEN,rot_val);	/* rotate value **/
#endif 							/* __UCRP_ROT_ENABLE__	**/
#ifdef __UCRP_TAB_ENABLE__		
 u_char tab_val;			/* tab **/
 tab_val =  ucrp_random();		/* **/
 ucrp_key_val(ucrp_tab_base,UCRP_TAB_LEN,tab_val);
#endif 							/* __UCRP_TAB_ENABLE__	**/
 u_char tab[256];
 for(pos = 0; pos < 256; ++pos) {
    tab[pos] = pos;
 }
 for(pos = 0; pos < 256 * 4; ++pos) {
    u_char ch0;
    u_char ch1;
    u_char vl0;
    u_char vl1;
    ch0 = ucrp_random();
    ch1 = ucrp_random();
    vl0 = tab[ch0];
    vl1 = tab[ch1];
    tab[ch0] = vl1;
    tab[ch1] = vl0;
 }  
 for(pos = 0; pos < 256; ++pos) {
    ucrp_key_val(ucrp_table_base + pos * 8, 8, tab[pos]);
 }
#if defined __UCRP_XOR_ENABLE__ || defined __UCRP_XRE_ENABLE__
 {				/* defined xor or xre algorithm **/
    u_char xrl;					/* xor low **/
    u_char xrv;					/* xor low value **/
    xrl = (ucrp_random() % (UCRP_LOW_MAX + 1));	/* random xor low **/
    xrv = (ucrp_random() & 0x7);		/* random xor low value **/
    ucrp_key_val(UCRP_XRL_BASE,UCRP_XRL_LEN, xrl); /* write xor low **/
    ucrp_key_val(UCRP_XRV_BASE,UCRP_XRV_LEN, xrv); /* write xor value **/
 }
#endif 							/* __UCRP_XOR_ENABLE__ **/
#ifdef __UCRP_BTS_ENABLE__
 {					/* fill bit table **/
    u_char bts_val;			/* **/
    bts_val = ucrp_random();		/* **/
    ucrp_key_val(ucrp_bts_base,UCRP_BTS_LEN,bts_val);
    u_char bttb[8];			/* bit swapping table **/
    int bpos;				/* bit swapping table position **/
    for(bpos = 0; bpos < 8; ++bpos) {	/* fill bit swapping table **/
	bttb[bpos] = bpos;		
    }
    for(bpos = 0; bpos < 16; ++bpos) {	    /* swap bit table **/
	u_char ch0;			/* table position 0 **/
	u_char ch1;			/* table position 1 **/
	u_char vl0;			/* readed value 0 **/
	u_char vl1;			/* readed value 1 **/
	ch0 = (ucrp_random() & 0x7);	/* random table position 0 **/
	ch1 = (ucrp_random() & 0x7);	/* random table position 1 **/
	vl0 = bttb[ch0];
	vl1 = bttb[ch1];
	bttb[ch0] = vl1;
	bttb[ch1] = vl0;
    }
    for(bpos = 0; bpos < 8; ++bpos) {	/* write bit swap table to key buffer **/
	ucrp_key_val(ucrp_bts_table_base + bpos * 8, 8, bttb[bpos]);
    }
 }  
#endif							/* __UCRP_BTS_ENABLE__ **/
 return(1);				/* **/
}
int ucrp_comp_read(void)		/* read compatibylity buffer **/	
{
 if(ucrp_fi == 0) {			/* input file descriptor **/
    return(0);
 }
 if(fread(ucrp_comp_buf, 1, sizeof(ucrp_comp_buf), ucrp_fi) != 
    sizeof(ucrp_comp_buf)) {
    return(0);
 }
 ucrp_src_buf_size = 
    ucrp_comp_val( 			/* write source buffer size to compatibility buffer **/
	UCRP_SBL, UCRP_SBL_LEN, -1);
 if(ucrp_src_buf_size == 0) {
    ucrp_src_buf_size = UCRP_SRC_SIZE;
 }
 ucrp_dst_buf_size = 
    ucrp_comp_val(			/* write destination buffer size to compatibility buffer **/
	UCRP_DBL, UCRP_DBL_LEN, -1);
 if(ucrp_dst_buf_size == 0) {
    ucrp_dst_buf_size = UCRP_BUF_SIZE;
 }
 return(1);
}
int ucrp_comp_write(void)		/* read compatibylity buffer **/
{
 if(ucrp_fo == 0) {			/* input file descriptor **/
    return(0);
 }
 if(fwrite(ucrp_comp_buf, 1, sizeof(ucrp_comp_buf), ucrp_fo) != 
    sizeof(ucrp_comp_buf)) {
    return(0);
 }
 return(1);
}
int ucrp_key_read(void)			/* read key buffer **/
{
 if(ucrp_fi == 0) {			/* input file descriptor **/
    return(0);
 }
 if(fread(ucrp_key_buf, 1, sizeof(ucrp_key_buf), ucrp_fi) != 
    sizeof(ucrp_key_buf)) {
    return(0);
 }
 return(1);
}
int ucrp_key_write(void)
{
 if(ucrp_fo == 0) {			/* output file descriptor **/
    return(0);
 }
 if(fwrite(ucrp_key_buf, 1, sizeof(ucrp_key_buf), ucrp_fo) != 
    sizeof(ucrp_key_buf)) {
    return(0);
 }
 return(1);
}
int ucrp_info(void)			/* print file information **/
{
 {					/* print ucrp version **/
    char cver[9];			/* ver **/
    int vpos;				/* buffer position **/
    for(vpos = 0; vpos < 9; ++vpos) { 	
	cver[vpos] = ucrp_comp_buf[vpos];
    }
    cver[8] = '\0';			/* string terminator **/
    ucrp_printf(UMSG_INFO_VERSION, cver); /* print program version **/
 }
#if defined __UCRP_XOR_ENABLE__
 if(ucrp_passwd[0] == '\0') {
    u_char xor_comp;			/* xor **/
    u_char xor_val;			/* xor **/
    xor_comp = ucrp_comp(ucrp_xor_comp, -1);	/* xor compatibility read **/
    xor_val = ucrp_key_val(UCRP_XOR_BASE,UCRP_XOR_LEN,-1);
    ucrp_printf(UMSG_INFO_XOR, (xor_comp ? UMSG_INFO_ON : UMSG_INFO_OFF), xor_val);
    {
	u_char xrl;			/* xor low **/
	u_char xrv;			/* xor low value **/
        char * sxrv;			/* xor low string **/
	xrl = ucrp_key_val(UCRP_XRL_BASE, UCRP_XRL_LEN, -1); /* read low numeric value **/
	xrv = ucrp_key_val(UCRP_XRV_BASE, UCRP_XRV_LEN, -1); /* read low value **/	
	if(xrl != UCRP_LOW_NULL) {		 /* **/
	    if(xrv == UCRP_LOW_NULL) {	/* null low **/
		sxrv = UMSG_LOW_NULL;
	    } else
	    if(xrv == UCRP_LOW_AD3) {	/* add low **/
		sxrv = UMSG_LOW_AD3;  
	    } else
	    if(xrv == UCRP_LOW_ROT) {	/* rot low **/
		sxrv = UMSG_LOW_ROT; 
	    } else
	    if(xrv == ucrp_low_tab) {	/* tab low **/
		sxrv = UMSG_LOW_TAB; 
	    } else
	    if(xrv == ucrp_low_bts) {	/* bts low **/
		sxrv = UMSG_LOW_BTS; 
	    } else {
		sxrv = UMSG_LOW_NULL;
	    }
	} else {
	    sxrv = UMSG_LOW_NULL;
	}
	ucrp_printf(UMSG_INFO_XRL, sxrv, xrl);	/* print xor low name **/
	ucrp_printf(UMSG_INFO_XRV, xrv);	/* print xor low value **/	
    }
 } else {				/* not null password */
    u_char xor_comp;			/* xor **/
    u_char xor_val;			/* xor **/
    xor_comp = ucrp_comp(ucrp_xor_comp, -1);	/* xor compatibility **/
    xor_val = ucrp_key_val(UCRP_XOR_BASE, UCRP_XOR_LEN, -1) + ucrp_passwd[0];
    ucrp_printf(UMSG_INFO_XOR, (xor_comp ? UMSG_INFO_ON : UMSG_INFO_OFF), xor_val);
    if(xor_comp != 0) {
        char * sxrv;			/* xor low string **/
	u_char xrl;			/* xor low **/
	u_char xrv;			/* xor low value **/
	xrl = ucrp_key_val(UCRP_XRL_BASE, UCRP_XRL_LEN, -1); /* read low numeric value **/	
	xrv = ucrp_key_val(UCRP_XRV_BASE, UCRP_XRV_LEN, -1); /* read low value **/
        if(xrl != UCRP_LOW_NULL) {
	    xrl += (ucrp_passwd[0] >> 3);	/* modify xor low with passwd **/
	    xrl =  (xrl % (UCRP_LOW_MAX + 1));	/* correct modified value **/
	    if(xrv == UCRP_LOW_AD3) {		/* add low **/
		xrv = (xrv + ucrp_passwd[0]);	/* xor add value **/
		sxrv = UMSG_LOW_AD3;
	    } else 
	    if(xrv == UCRP_LOW_ROT) {		/* rot low **/
		xrv = ((xrv + ucrp_passwd[0]) & 0x7);	/* rotate value **/
		sxrv = UMSG_LOW_ROT;
	    } else 
	    if(xrv == ucrp_low_tab) {		/* tab low **/
		xrv = (xrv + ucrp_passwd[0]);	/* modify xor low value **/
		sxrv = UMSG_LOW_TAB;
	    } else 
	    if(xrv == ucrp_low_bts) {		/* bts low **/
		xrv = ((xrv + ucrp_passwd[0]) & 0x7);	/* modify xor low value **/
		sxrv = UMSG_LOW_BTS;
	    } else {
    		sxrv = UMSG_LOW_UNKN;		/* unknown low **/
	    }
	} else {
	    sxrv = UMSG_LOW_NULL;
	}
	ucrp_printf(UMSG_INFO_XRL, sxrv, xrl);	/* print xor low name **/
	ucrp_printf(UMSG_INFO_XRV, xrv);	/* print xor low value **/	
    }
 }
#endif					/* defined __UCRP_XOR_ENABLE__ **/
#if defined __UCRP_XRE_ENABLE__
 if(ucrp_passwd[0] == '\0') {
    u_char xre_comp;			/* xor **/
    u_char xor_val;			/* xor **/
    xre_comp = ucrp_comp(ucrp_xre_comp, -1); /* xor compatibility **/
    xor_val = ucrp_key_val(UCRP_XOR_BASE,UCRP_XOR_LEN,-1);
    ucrp_printf(UMSG_INFO_XRE, (xre_comp ? UMSG_INFO_ON : UMSG_INFO_OFF), xor_val);
    {
	u_char xrl;			/* xor low **/
	u_char xrv;			/* xor low value **/
        char * sxrv;			/* xor low string **/
	xrl = ucrp_key_val(UCRP_XRL_BASE, UCRP_XRL_LEN, -1); /* read low numeric value **/
	xrv = ucrp_key_val(UCRP_XRV_BASE, UCRP_XRV_LEN, -1); /* read low value **/	
	if(xrl == UCRP_LOW_NULL) {
	    sxrv = UMSG_LOW_NULL;	/* null low **/
	} else 
	if(xrl == UCRP_LOW_AD3) {
	    sxrv = UMSG_LOW_AD3;	/* add low **/
	} else 
	if(xrl == UCRP_LOW_ROT) {
	    sxrv = UMSG_LOW_ROT;	/* rot low **/
	} else
	if(xrl == ucrp_low_tab) {
	    sxrv = UMSG_LOW_TAB;	/* tab low **/
	} else
	if(xrl == ucrp_low_bts) {
	    sxrv = UMSG_LOW_BTS;	/* bts low **/
	} else {
	    sxrv = UMSG_LOW_UNKN;
	}
	ucrp_printf(UMSG_INFO_XRL, sxrv, xrl);	/* print xor low name **/
	ucrp_printf(UMSG_INFO_XRV, xrv);	/* print xor low value **/	
    }
 } else {
    u_char xre_comp;			/* xor **/
    u_char xor_val;			/* xor **/
    xre_comp = ucrp_comp(ucrp_xre_comp, -1);	/* xre compatibility **/
    if(xre_comp != 0) {
        char * sxrv;			/* xor low string **/
	u_char xrl;			/* xor low **/
	u_char xrv;			/* xor low value **/
	xor_val = ucrp_key_val(UCRP_XOR_BASE, UCRP_XOR_LEN, -1) + 
		ucrp_passwd[0];
	xrl = ucrp_key_val(UCRP_XRL_BASE, UCRP_XRL_LEN, -1); /* read low numeric value **/
	xrv = ucrp_key_val(UCRP_XRV_BASE, UCRP_XRV_LEN, -1); /* read low value **/
	xrl += (ucrp_passwd[0] >> 3);		/* modify xor low with passwd **/
	xrl = (xrl % (UCRP_LOW_MAX + 1));	/* correct modified value **/
	xrv = ((xrv + ucrp_passwd[0]) & 0x7);	/* rotate value **/
	if(xrl != UCRP_LOW_NULL) {
	    if(xrl == UCRP_LOW_AD3) {	/* add low **/	
		sxrv = UMSG_LOW_AD3;		
	    } else
	    if(xrl == UCRP_LOW_ROT) {	/* rot low **/
		sxrv = UMSG_LOW_ROT;
	    } else 
	    if(xrl == ucrp_low_tab) {	/* tab low **/
		sxrv = UMSG_LOW_TAB;
	    } else
	    if(xrl == ucrp_low_bts) {	/* bts low **/
		sxrv = UMSG_LOW_BTS;
	    } else {
		sxrv = UMSG_LOW_UNKN;	/* unknown low **/
	    }
	} else {
	    sxrv = UMSG_LOW_NULL;
	}
	ucrp_printf(UMSG_INFO_XRE, (xre_comp ? UMSG_INFO_ON : UMSG_INFO_OFF), xor_val);
	ucrp_printf(UMSG_INFO_XRL, sxrv, xrl);	/* print xor low name **/
	ucrp_printf(UMSG_INFO_XRV, xrv);	/* print xor low value **/	
    }
 }
#endif 					/*  defined __UCRP_XRE_ENABLE__ **/
#ifdef __UCRP_ADD_ENABLE__
 {
    u_char add_comp;			/* add **/
    u_char add_val;			/* add **/
    add_comp = ucrp_comp(ucrp_add_comp, -1);	/* add compatibility **/
    if(ucrp_passwd[0] == '\0') {
	add_val = ucrp_key_val(UCRP_ADD_BASE,UCRP_ADD_LEN,-1);
    } else {
	add_val = ucrp_key_val(UCRP_ADD_BASE,UCRP_ADD_LEN,-1) + ucrp_passwd[1];
    }
    ucrp_printf(UMSG_INFO_ADD, (add_comp ? UMSG_INFO_ON : UMSG_INFO_OFF), add_val);
 }
#endif 						/* __UCRP_ADD_ENABLE__ **/
#ifdef __UCRP_ROT_ENABLE__
 {
    u_char rot_comp;			/* rot **/
    u_char rot_val;			/* rot **/
    rot_comp = ucrp_comp(ucrp_rot_comp, -1);	/* byte rotate **/
    if(ucrp_passwd[0] == '\0') {
	rot_val = ucrp_key_val(UCRP_ROT_BASE,UCRP_ROT_LEN,-1);
    } else {
	rot_val = (ucrp_key_val(UCRP_ROT_BASE,UCRP_ROT_LEN,-1) + ucrp_passwd[2]) & 0x7;
    }
    ucrp_printf(UMSG_INFO_ROT, (rot_comp ? UMSG_INFO_ON : UMSG_INFO_OFF), rot_val);
 }
#endif 						/* __UCRP_ROT_ENABLE__ **/
#ifdef __UCRP_TAB_ENABLE__
 {
    u_char tab_comp;			/* tab **/
    u_char tab_val;			/* tab **/
    tab_comp = ucrp_comp(ucrp_tab_comp, -1);		/* byte table **/
    tab_val = ucrp_key_val(ucrp_tab_base,UCRP_TAB_LEN,-1);
    ucrp_printf(UMSG_INFO_TAB, (tab_comp ? UMSG_INFO_ON : UMSG_INFO_OFF), tab_val);
    {
	u_char ttab[256];				/* translate table **/
	int tpos;					/* translate positiom **/
	for(tpos = 0; tpos < 256; ++tpos) {	
    	    ttab[tpos] = ucrp_key_val(	/* read translate table **/
		ucrp_table_base + (tpos * 8), 8, -1);
	}
	ucrp_printf(UMSG_INFO_TRT);
	for(tpos = 0; tpos < 256; ++tpos) {	/* print translate table value **/
    	    ucrp_printf(UMSG_02X, ttab[tpos]);	/* translate table value **/	
    	    ucrp_printf(UMSG_SP);			/* translate table value **/	
	    if(((tpos + 1) & 0xF) == 0) {	
    		ucrp_printf(UMSG_CRNL);		/* print table separator **/
	    }
	}
    }
 }
#endif 						/* __UCRP_TAB_ENABLE__ **/
#ifdef __UCRP_ECC_ENABLE__
 {
    u_char ecc_comp;			/* ecc **/
    ecc_comp = ucrp_comp(ucrp_ecc_comp, -1);		/* ecc coding **/	
    ucrp_printf(UMSG_INFO_ECC, (ecc_comp ? UMSG_INFO_ON : UMSG_INFO_OFF));
 }
#endif 						/* __UCRP_ECC_ENABLE__ **/
#ifdef __UCRP_BTS_ENABLE__
 {
    u_char bts_comp;			/* bts compativility **/
    u_int8_t bts_val;			/* bts value */
    u_char bts_tab[8];			/* bts conversion table **/
    bts_comp = ucrp_comp(ucrp_bts_comp, -1);	/* read bts compatibility **/	
    bts_val = ucrp_key_val(ucrp_bts_base,UCRP_BTS_LEN,-1);
    {
	ucrp_printf(UMSG_INFO_BTS, (bts_comp ? UMSG_INFO_ON : UMSG_INFO_OFF), bts_val);
	/* write bit swap table to key buffer **/
	int bpos;				/* bts table position **/
	for(bpos = 0; bpos < 8; ++bpos) {	/* read bts value **/
	    bts_tab[bpos] = 
		ucrp_key_val(ucrp_bts_table_base + bpos * 8, 8, -1);
	}
	for(bpos = 0; bpos < 8; ++bpos) {	/* read bts value **/
	    ucrp_printf(UMSG_02X, bts_tab[bpos]);
	    ucrp_printf(UMSG_SP);
	}
	ucrp_printf(UMSG_CRNL);		/* print table separator **/
    }
 }
#endif 						/* __UCRP_BTS_ENABLE__ **/
 int ssize;				/* source buffer size **/
 ssize = ucrp_comp_val(UCRP_SBL, UCRP_SBL_LEN, -1);
 ucrp_printf(UMSG_INFO_SBL, ssize, ssize);

 int dsize;				/* destination buffer size **/
 dsize = ucrp_comp_val(UCRP_DBL, UCRP_DBL_LEN, -1);
 ucrp_printf(UMSG_INFO_DBL, dsize, dsize);
 return(1);				/* **/
}
int ucrp_hash_init(void)		/* passwd hash init**/
{
 int rvl;				/* return code */
 rvl = 1;				/* start return code */
 u_int32_t pos;
 if(ucrp_passwd[0] == '\0') {		/* empty passwd **/
    for(pos = 0; pos < sizeof(ucrp_passwd); ++pos) {
	if(ucrp_comp_version() >= 118) {
	    ucrp_passwd[pos] = 0;
	} else {
	    ucrp_passwd[pos] = pos;
	}
    }
 } else {				/* not empty passwd **/
    int len;				/* passwd length **/
    len = strlen(ucrp_passwd);		/* read passwd length **/ 
    pos = len;				/* to passwd end **/
    while(pos < sizeof(ucrp_passwd)) {
	int dpos;
	dpos = 0;
	while(dpos < len) {
	    if(pos >= sizeof(ucrp_passwd)) {
		break;
	    }
	    ucrp_passwd[pos] = ucrp_passwd[dpos];
	    dpos += 1;
	    pos += 1;
	}
    }
 }
#if defined __UCRP_XOR_ENABLE__ || defined __UCRP_XRE_ENABLE__ 
 u_char xor0_comp;			/* xor basic compatibility **/
 u_char xor1_comp;			/* xor extended compatibility **/
 u_char xor_comp;			/* xor compatibility **/
 u_char xor_val;			/* xor value **/
 xor0_comp =
#if defined __UCRP_XOR_ENABLE__
    ucrp_comp(ucrp_xor_comp, -1);	/* xor compatibility **/
#else 
    0;
#endif 							/* __UCRP_XOR_ENABLE__ **/
 xor1_comp =
#if defined __UCRP_XRE_ENABLE__
    ucrp_comp(ucrp_xre_comp, -1);	/* xre compatibility **/
#else 
    0;
#endif 							/* __UCRP_XOR_ENABLE__ **/
 if((xor0_comp == 1) || (xor1_comp == 1)) {
/* defined xor basic or xor extended algorithm **/
    xor_comp = 1;
 } else {
    xor_comp = 0;			/* disable any xor algorithm **/
 }
 xor_val = ucrp_key_val(UCRP_XOR_BASE,UCRP_XOR_LEN,-1);
 if(xor_comp) {				/* xor compatibility **/
    for(pos = 0; pos < sizeof(ucrp_passwd); ++pos) {
	u_char rch;
	rch = ucrp_passwd[pos];
	rch = (rch ^ xor_val);
	ucrp_passwd[pos] = rch;
    }
 }
#endif				/* __UCRP_XOR_ENABLE__ || __UCRP_XRE_ENABLE__ */
#ifdef __UCRP_ADD_ENABLE__
 u_char add_comp;			/* add compatibility **/
 u_char add_val;			/* add value **/
 add_comp = ucrp_comp(ucrp_add_comp, -1); /* add compatibility **/
 add_val = ucrp_key_val(UCRP_ADD_BASE,UCRP_ADD_LEN,-1);
 if(add_comp) {				/* add compatibility **/
    for(pos = 0; pos < sizeof(ucrp_passwd); ++pos) {
	u_char rch;
	rch = ucrp_passwd[pos];
	rch = (rch + add_val);
	ucrp_passwd[pos] = rch;
    }
 }
#endif						/* __UCRP_ADD_ENABLE__ */
#ifdef __UCRP_ROT_ENABLE__
 u_char rot_comp;			/* rot compatibility **/
 u_char rot_val;			/* rot value **/
 rot_comp = ucrp_comp(ucrp_rot_comp, -1); /* byte rotate **/
 rot_val = ucrp_key_val(UCRP_ROT_BASE,UCRP_ROT_LEN,-1);
 if(rot_comp) {				/* rot compatibility **/
    for(pos = 0; pos < sizeof(ucrp_passwd); ++pos) {
	u_char rch;
	rch = ucrp_passwd[pos];
	if(ucrp_comp_version() >= 118) {
	    rch = ((rch << rot_val) | (rch >> (8 - rot_val)));
	} else {
	    rch = ((rch >> rot_val) | (rch << (8 - rot_val)));
	}
	ucrp_passwd[pos] = rch;
    }
 }
#endif						/* __UCRP_ROT_ENABLE__ */
#ifdef __UCRP_TAB_ENABLE__
 u_char tab_comp;			/* tab compatibility **/
 u_char tab_val;			/* tab value **/
 tab_comp = ucrp_comp(ucrp_tab_comp, -1); /* byte table **/
 tab_val = ucrp_key_val(ucrp_tab_base,UCRP_TAB_LEN,-1);
 if(tab_comp) {
    u_char tab[256];
    u_int32_t dpos;			/* table destination position */
    if(ucrp_comp_version() >= 118) {	/* ucrp compatibility test */
        dpos = (tab_val & 0xFF);	/* destination position start value */
    } else {
	dpos = 0;			/* destination position start value */
    }
    for(pos = 0; pos < 256; ++pos) {
	if(ucrp_comp_version() >= 118) {	/* ucrp compatibility test */
	    tab[dpos] = ucrp_key_val(ucrp_table_base + pos * 8, 8, -1);
	} else {
	    tab[dpos] = pos;
	}
	dpos += 1;
	if(dpos >= 256) {
	    dpos = 0;
	}
    }
#if defined __UCRP_XOR_ENABLE__ || defined __UCRP_XRE_ENABLE__
    if(xor_comp) {
	for(pos = 0; pos < 256; ++pos) {
	    u_char ch;
	    ch = tab[pos];
	    ch = (ch ^ xor_val);
	    tab[pos] = ch;
	}
    }
#endif			/* __UCRP_XOR_ENABLE__ || __UCRP_XRE_ENABLE__ */
#ifdef __UCRP_ADD_ENABLE__
    if(add_comp) {
	for(pos = 0; pos < 256; ++pos) {
	    u_char ch;
	    ch = tab[pos];
	    ch = (ch + add_val);
	    tab[pos] = ch;
	}
    }
#endif						/* __UCRP_ADD_ENABLE__ */
#ifdef __UCRP_ROT_ENABLE__
    if(rot_comp) {
	for(pos = 0; pos < 256; ++pos) {
	    u_char ch;
	    ch = tab[pos];
	    if(ucrp_comp_version() >= 118) {	/* ucrp compatibility test */
	        ch = ((ch << rot_val) | (ch >> (8 - rot_val)));
	    } else {
	        ch = ((ch >> rot_val) | (ch << (8 - rot_val)));
	    }
	    tab[pos] = ch;
	}
    }
#endif						/* __UCRP_ROT_ENABLE__ */
    for(pos = 0; pos < sizeof(ucrp_passwd); ++pos) {
	u_char ch;
	ch = ucrp_passwd[pos];
	ch = tab[ch];
	ucrp_passwd[pos] = ch;
    }
 }
#endif						/* __UCRP_TAB_ENABLE__ */
#ifdef __UCRP_BTS_ENABLE__
 if(ucrp_comp_version() >= 118) {	/* ucrp compatibility test */
    u_char bts_comp;			/* tab compatibility **/
    u_char bts_val;			/* tab value **/
    bts_comp = ucrp_comp(ucrp_bts_comp, -1); /* byte table **/
    bts_val = ucrp_key_val(ucrp_bts_base,UCRP_BTS_LEN,-1);
    if(bts_comp) {				/* bts compatibility test */
	u_int8_t bttb[8];			/* bit swapping table **/
	u_int8_t tval;				/* bit swapping check table **/
	int dpos;				/* bit swapping tavle position  **/
	tval = 0;				/* test value **/
	dpos = (bts_val & 0x7);			/* table position **/
	for(pos = 0; pos < 8; ++pos) {		/* read bit swapping table **/
	    u_int8_t chky;
    	    chky = ucrp_key_val(ucrp_bts_table_base + pos * 8, 8, -1);
	    if(chky >= 8) {			/* not correct value for bit number **/
		rvl = 0;
	    }
	    if((tval & (1 << chky)) > 0) {	
		rvl = 0;
	    } else {
		tval |= (1 << chky);		/* set bit **/
	    }
	    bttb[dpos] = chky;			/* write to bit swapping table **/
	    dpos = ((dpos + 1) & 0x7);		/* next key read position **/
	}
	u_int8_t btab[256];
        int tpos;				/* translate table position **/
	for(tpos = 0; tpos < 256; ++tpos) {	
	    u_int8_t tval = tpos;		/* translate table index **/
	    u_int8_t tres = 0;			/* translate value **/
	    for(dpos = 0; dpos < 8; ++dpos) {		
		if((tval & (1 << bttb[dpos]))) { /* if bit seted in inde **/
		    tres |= (1 << dpos);	/* set bit in translated value  **/
    		}
	    }
	    btab[tpos] = tres;		/* write to result value **/
	}
	for(pos = 0; pos < sizeof(ucrp_passwd); ++pos) {
	    u_char rch;
	    rch = ucrp_passwd[pos];
	    rch = btab[rch];
	    ucrp_passwd[pos] = rch;
	}
    }
 }
#endif						/* __UCRP_BTS_ENABLE__ */
#ifdef __UCRP_HASH_DEBUG__
 ucrp_printf(UMSG_CRLF);
 ucrp_printf(UMSG_HASH);
 for(pos = 0; pos < (int)sizeof(ucrp_passwd); ++pos) {
    ucrp_printf(UMSG_02X, ucrp_passwd[pos]);
    ucrp_printf(UMSG_SP);
 }
 ucrp_printf(UMSG_CRLF);
#endif 							/* __UCRP_HASH_DEBUG__ **/
 return(rvl);
}
int ucrp_read(u_char * src, int cnt)	/* read buffer from input file **/
{
 if(
    (ucrp_fi == 0) ||			/* input file pointer check */
    (src == 0)				/* source buffer pointer check */
 ) {
    return(0);
 }
 cnt = fread(src, 1, cnt, ucrp_fi);	/* read file **/
 return(cnt);
}
int ucrp_write(u_char * src, int cnt)
{
 if(
    (ucrp_fi == 0) ||			/* input file pointer check */
    (src == 0)				/* source buffer pointer check */
 ) {
    return(0);
 }
 cnt = fwrite(src, 1, cnt, ucrp_fo);	
 return(cnt);
}
#ifdef __UCRP_XOR_ENABLE__
int ucrp_xor_en(u_char * dst, u_char * src, int len) /* xor encrypt **/
{
 if(
    (dst == 0) ||			/* destination buffer pointer check */
    (src == 0)				/* source buffer pointer check */
 ) {
    return(0);
 }
 u_char xor_val;			/* xor mask **/
 u_char xrl;				/* xor mask change low **/
 u_char xrv;				/* xor mask change low value **/
 int rvl; 				/* return value **/
 rvl = 1;				/* start return value **/
 xor_val = ucrp_key_val(UCRP_XOR_BASE, UCRP_XOR_LEN, -1) + ucrp_passwd[0];
 xrl = ucrp_key_val(UCRP_XRL_BASE, UCRP_XRL_LEN, -1); /* read low numeric value **/
 xrv = ucrp_key_val(UCRP_XRV_BASE, UCRP_XRV_LEN, -1); /* read low value **/
 if(xrl != UCRP_LOW_NULL) {		 /* **/
    xrl += (ucrp_passwd[0] >> 3);	/* modify xor low with passwd **/
    xrl =  (xrl % (UCRP_LOW_MAX + 1));	/* correct modified value **/
    if(xrv == UCRP_LOW_AD3) {		/* add low **/
	xrv = (xrv + ucrp_passwd[0]);	/* xor add value **/
	int pos;
	for(pos = 0; pos < len; ++pos) {
	    *(dst + pos) = (*(src + pos) ^ xor_val); /* encryp/decrypt byte **/	
	    xor_val += xrv;		/* modify xor mask **/
	}
    } else 
    if(xrv == UCRP_LOW_ROT) {		/* rot low **/
	xrv = ((xrv + ucrp_passwd[0]) & 0x7);	/* rotate value **/
	int pos;
	for(pos = 0; pos < len; ++pos) {
	    *(dst + pos) = (*(src + pos) ^ xor_val);	/* encrypt/decrypt byte **/
	    xor_val = (xor_val >> xrv) | (xor_val << (8 - xrv)); /* rotate xor mask **/
	}
    } else 
    if(xrv == ucrp_low_tab) {		/* tab low **/
	u_char ttab[256];		/* mask change table **/
	int spos;			/* source buffer position **/
	int dpos;			/* destination buffer position **/
	xrv = (xrv + ucrp_passwd[0]);	/* modify xor low value **/
	dpos = (xrv & 0xFF);		/* correct xor low value **/
	for(spos = 0; spos < 256; ++spos) { /* read mask translate table **/		
	    ttab[spos] = ucrp_key_val(ucrp_table_base + (dpos * 8), 8, -1);
	    dpos += 1;			/* next position in destination table **/
	    if(dpos >= 256) {		/* set destination position to start of tranlate table **/
		dpos = 0;
	    }
	}
	int pos;			
	for(pos = 0; pos < len; ++pos) {
	    *(dst + pos) = (*(src + pos) ^ xor_val);	/* encrypt/decryp byte **/
	    xor_val = ttab[ (xor_val & 0xFF) ];	/* modify xor mask **/
	}
    } else 
    if(xrv == ucrp_low_bts) {		/* bts low **/
	u_char bttb[8];			/* write to bit swapping table **/
	int dpos;			/* next key read position **/
	int bpos;			/* bit table position **/
	int tval;			/* test value **/
	xrv = ((xrv + ucrp_passwd[0]) & 0x7); /* modify xor low value **/
	dpos = xrv;			/* destination position **/
	if((dpos < 0) || (dpos >= 8)) {	/* set destination position **/
	    dpos = 0;			/* to buffer start **/
	}
	tval = 0;			/* test value **/
	for(bpos = 0; bpos < 8; ++bpos) {
	    u_int8_t chky;		/* readed byte from bit swapping table **/
	    chky = ucrp_key_val(ucrp_bts_table_base + dpos * 8, 8, -1); /* read from table **/
	    if(chky >= 8) {		/* not correct bit number **/
	    	chky = 0;		/* 0 - 7 **/	
		rvl = 0;		/* clear return value **/
	    }
	    if((tval & (1 << chky)) > 0) {	
		rvl = 0;		/* internal error -> clear return value **/	
	    } else {
		tval |= (1 << chky);		/* set bit **/
	    }
	    bttb[dpos] = chky;		/* write to bit swapping table **/
	    dpos = ((dpos + 1) & 0x7);	/* next key read position **/
	}
	if(rvl == 0) {
	    ucrp_error(UMSG_XOR_ERR);
	}
	u_char ttab[256];		/* translate table **/
	int tpos;			/* translate table position **/
	for(tpos = 0; tpos < 256; ++tpos) {	
	    u_int8_t tval = tpos;	/* translate table index **/
	    u_int8_t tres = 0;		/* translate value **/
	    for(bpos = 0; bpos < 8; ++bpos) {		
		if( (tval & (1 << bttb[bpos])) ) {  /* if bit seted in inde **/
		    tres |= (1 << bpos);    /* set bit in translated value  **/
		}
	    }
	    *(ttab + tpos) = tres;	/* write to result value **/
	}
	int pos;
	for(pos = 0; pos < len; ++pos) {
	    *(dst + pos) = (*(src + pos) ^ xor_val); /* encrypt/decrypt byte **/
	    xor_val = ttab[ xor_val ];	/* modify xor mask **/
	}
    } else {				/* unknown low **/
	int pos;
	for(pos = 0; pos < len; ++pos) {
	    /* encrypt/decrypt byte **/
	    *(dst + pos) = (*(src + pos) ^ xor_val);
	}
    }
 } else {
/* UCRP_LOW_NULL - xor with constant mask **/
    int pos;				/* buffer position **/
    for(pos = 0; pos < len; ++pos) {	
	*(dst + pos) = (*(src + pos) ^ xor_val);	/* encrypt byte **/
    }
 }
 return(len);				/* bytes encrypted/decrypted **/
}
#endif						/* __UCRP_XOR_ENABLE__ **/
#ifdef __UCRP_XRE_ENABLE__
/* xor with extended compatibility **/
int ucrp_xre_en(u_char * dst, u_char * src, int len)	/* xor encrypt **/
{
 if(
    (dst == 0) ||			/* destination buffer pointer check */
    (src == 0)				/* source buffer pointer check */
 ) {
    return(0);
 }
 u_char xor_val;			/* xor mask **/
 u_char xrl;				/* xor mask change low **/
 u_char xrv;				/* xor mask change low value **/
 int rvl; 				/* return value **/
 rvl = 1;				/* start return value **/
 xor_val = (ucrp_key_val(UCRP_XOR_BASE, UCRP_XOR_LEN, -1) + ucrp_passwd[0]);
 xrl = ucrp_key_val(UCRP_XRL_BASE, UCRP_XRL_LEN, -1); /* read low numeric value **/
 xrv = ucrp_key_val(UCRP_XRV_BASE, UCRP_XRV_LEN, -1); /* read low value **/
 xrl += (ucrp_passwd[0] >> 3);		/* modify xor low with passwd **/
 xrl = (xrl % (UCRP_LOW_MAX + 1));	/* correct modified value **/
 if(xrl != UCRP_LOW_NULL) {
    if(xrl == UCRP_LOW_AD3) {		/* add low **/
        if(ucrp_comp_version() >= 118) {
	    xrv = ((xrv + ucrp_passwd[0]) & 0xFF); /* rotate value **/
	} else {
	    xrv = ((xrv + ucrp_passwd[0]) & 0x7); /* rotate value **/
	}	
	int pos;
	for(pos = 0; pos < len; ++pos) {
	    *(dst + pos) = (*(src + pos) ^ xor_val); /* encryp/decrypt byte **/
	    xor_val += xrv;		/* modify xor mask **/
	}
    } else 
    if(xrl == UCRP_LOW_ROT) {		/* rot low **/
	xrv = ((xrv + ucrp_passwd[0]) & 0x7);	/* rotate value **/
	int pos;
	for(pos = 0; pos < len; ++pos) {
	    *(dst + pos) = (*(src + pos) ^ xor_val); /* encrypt/decrypt byte **/
	    if(ucrp_comp_version() >= 118) {
		xor_val = (xor_val << xrv) | (xor_val >> (8 - xrv)); /* rotate xor mask **/
	    } else {
		xor_val = (xor_val >> xrv) | (xor_val << (8 - xrv)); /* rotate xor mask **/
	    }
	}
    } else 
    if(xrl == ucrp_low_tab) {		/* tab low **/
	if(ucrp_comp_version() >= 118) {
	    xrv = ((xrv + ucrp_passwd[0]) & 0xFF); /* rotate value **/
	} else {
	    xrv = ((xrv + ucrp_passwd[0]) & 0x7); /* rotate value **/
	}
	u_char ttab[256];		/* mask change table **/
	int spos;			/* source buffer position **/
	int dpos;			/* destination buffer position **/
	dpos = xrv;			/* correct xor low value **/
	for(spos = 0; spos < 256; ++spos) { 	/* read mask translate table **/	 
	    if(ucrp_comp_version() >= 118) {
	        ttab[dpos] = ucrp_key_val(ucrp_table_base + (spos * 8), 8, -1);
	    } else {
	        ttab[spos] = ucrp_key_val(ucrp_table_base + (dpos * 8), 8, -1);
	    }
	    dpos += 1;			/* next position in destination table **/
	    if(dpos >= 256) {		/* set destination position to start of tranlate table **/
		dpos = 0;
	    }
	}
	int pos;			
	for(pos = 0; pos < len; ++pos) {
	    *(dst + pos) = (*(src + pos) ^ xor_val); /* encrypt/decryp byte **/
	    xor_val = ttab[ (xor_val & 0xFF) ]; /* modify xor mask **/
	}
    } else 
    if(xrl == ucrp_low_bts) {		/* bts low **/
	xrv = ((xrv + ucrp_passwd[0]) & 0x7); /* rotate value **/
	u_char bttb[8];			/* write to bit swapping table **/
	int dpos;			/* next key read position **/
	int bpos;			/* bit table position **/
	int tval;			/* test value **/
	dpos = xrv;			/* destination position **/
	if((dpos < 0) || (dpos >= 8)) {	/* set destination position **/
	    dpos = 0;			/* to buffer start **/
	}
	tval = 0;			/* test value **/
	for(bpos = 0; bpos < 8; ++bpos) {
	    u_int8_t chky;		/* readed byte from bit swapping table **/
	    if(ucrp_comp_version() >= 118) {
		chky = ucrp_key_val(ucrp_bts_table_base + bpos * 8, 8, -1); /* read from table **/
	    } else {
		chky = ucrp_key_val(ucrp_bts_table_base + dpos * 8, 8, -1); /* read from table **/
	    }
	    if(chky >= 8) {		/* not correct bit number **/
	    	chky = 0;		/* 0 - 7 **/	
		rvl = 0;		/* clear return value **/
	    }
	    if((tval & (1 << chky)) > 0) {	
	    /* internal error -> clear return value **/	
		rvl = 0;
	    } else {
		tval |= (1 << chky);		/* set bit **/
	    }
	    bttb[dpos] = chky;		/* write to bit swapping table **/
	    dpos = ((dpos + 1) & 0x7);	/* next key read position **/
	}
	if(rvl == 0) {
	    ucrp_error(UMSG_XOR_ERR);
	}
	u_char ttab[256];		/* translate table **/
	int tpos;			/* translate table position **/
	for(tpos = 0; tpos < 256; ++tpos) {	
	    u_int8_t tval = tpos;	/* translate table index **/
	    u_int8_t tres = 0;		/* translate value **/
	    for(bpos = 0; bpos < 8; ++bpos) {		
		if( (tval & (1 << bttb[bpos])) ) {  /* if bit seted in inde **/
		    tres |= (1 << bpos);	    /* set bit in translated value  **/
		}
	    }
	    *(ttab + tpos) = tres;	/* write to result value **/
	}
	int pos;
	for(pos = 0; pos < len; ++pos) {
	    *(dst + pos) = (*(src + pos) ^ xor_val); /* encrypt/decrypt byte **/
	    xor_val = ttab[ xor_val ];	/* modify xor mask **/
	}
    } else {
    /* unknown low **/
	int pos;
	for(pos = 0; pos < len; ++pos) { 
	    *(dst + pos) = (*(src + pos) ^ xor_val); /* encrypt/decrypt byte **/
	}
    }
 } else {
/* UCRP_LOW_NULL - xor with constant mask **/
    int pos;				/* buffer position **/
    for(pos = 0; pos < len; ++pos) {	
	*(dst + pos) = (*(src + pos) ^ xor_val);	/* encrypt byte **/
    }
 }
 if(rvl == 0) {
    return(-1);
 } else {
    return(len);			/* bytes encrypted/decrypted **/
 }
}
#endif						/* __UCRP_XRE_ENABLE__ **/
#ifdef __UCRP_ADD_ENABLE__
int ucrp_add_en(u_char * dst, u_char * src, int len) /* add encrypt **/
{
 if(
    (dst == 0) ||			/* destination buffer pointer check */
    (src == 0)				/* source buffer pointer check */
 ) {
    return(0);
 }
 u_char add_val;
 int pos;
 add_val = ucrp_key_val(UCRP_ADD_BASE,UCRP_ADD_LEN,-1) + ucrp_passwd[1];
 for(pos = 0; pos < len; ++pos) {
    u_char ch;
    ch = *(src + pos);
    ch = ch + add_val;
    *(dst + pos) = ch;
    if(ucrp_comp_version() >= 118) {
    } else {
	add_val += 1;
    }
 }
 return(len);
}
#endif 						/* __UCRP_ADD_ENABLE__ **/
#ifdef __UCRP_ADD_ENABLE__
int ucrp_add_de(u_char * dst, u_char * src, int len) /* add decrypt **/
{
 if(
    (dst == 0) ||			/* destination buffer pointer check */
    (src == 0)				/* source buffer pointer check */
 ) {
    return(0);
 }
 u_char add_val;
 int pos;
 add_val = ucrp_key_val(UCRP_ADD_BASE,UCRP_ADD_LEN,-1) + ucrp_passwd[1];
 for(pos = 0; pos < len; ++pos) {
    u_char ch;
    ch = *(src + pos);
    ch = (256 + ch) - add_val;
    *(dst + pos) = ch;
    if(ucrp_comp_version() >= 118) {
    } else {
	add_val += 1;
    }
 }
 return(len);
}
#endif 						/* __UCRP_ADD_ENABLE__ **/
#ifdef __UCRP_ROT_ENABLE__
int ucrp_rot_en(u_char * dst, u_char * src, int len)
{
 u_char rot_val;
 int pos;					/* **/
 rot_val = (
    (ucrp_key_val(UCRP_ROT_BASE,UCRP_ROT_LEN,-1) + ucrp_passwd[2]) & 0x7
 );
 for(pos = 0; pos < len; ++pos) {
    u_char ch;
    ch = *(src + pos);
    if(ucrp_comp_version() >= 118) {
	ch = ((ch << rot_val) | (ch >> (8 - rot_val)));
    } else {
	ch = ((ch >> rot_val) | (ch << (8 - rot_val)));
    }
    *(dst + pos) = ch;
 }
 return(len);
}
#endif 						/* __UCRP_ROT_ENABLE__ **/
#ifdef __UCRP_ROT_ENABLE__
int ucrp_rot_de(u_char * dst, u_char * src, int len)
{
 u_char rot_val;
 int pos;				/* **/
 rot_val = (
    (ucrp_key_val(UCRP_ROT_BASE,UCRP_ROT_LEN,-1) + ucrp_passwd[2]) & 0x7
 );
 for(pos = 0; pos < len; ++pos) {
    u_char ch;
    ch = *(src + pos);
    if(ucrp_comp_version() >= 118) {
	ch = ((ch >> rot_val) | (ch << (8 - rot_val)));
    } else {
	ch = ((ch << rot_val) | (ch >> (8 - rot_val)));
    }
    *(dst + pos) = ch;
 }
 return(len);
}
#endif 						/* __UCRP_ROT_ENABLE__ **/
int ucrp_bts_init(u_int8_t * ttab, int en, int pas) /* init bit swapping translate table **/
{
 u_int8_t bttb[8];			/* bit swapping table **/
 u_int8_t tval;				/* bit swapping check table **/
 int bpos;				/* bit swapping tavle position  **/
 int kpos;				/* key buffer position **/
 tval = 0;				/* test value **/
 if(ucrp_comp_version() >= 118) {
    if(pas > 0) { 
	kpos = (
	    (ucrp_key_val(ucrp_bts_base,UCRP_BTS_LEN,-1) + ucrp_passwd[3]) & 0x7
	);
    } else {
	kpos = (
	    ucrp_key_val(ucrp_bts_base,UCRP_BTS_LEN,-1) & 0x7
	);
    }
 } else {
    kpos = (ucrp_passwd[3] & 0x7);		/* table position **/
 }
 for(bpos = 0; bpos < 8; ++bpos) {	/* read bit swapping table **/
    u_int8_t chky;
    if(ucrp_comp_version() >= 118) {
	chky = ucrp_key_val(ucrp_bts_table_base + bpos * 8, 8, -1); /* read from table **/
    } else {
	chky = ucrp_key_val(ucrp_bts_table_base + kpos * 8, 8, -1); /* read from table **/
    }
    if(chky >= 8) {			/* not correct value for bit number **/
	return(0);
    }
    if((tval & (1 << chky)) > 0) {	
	return(0);
    } else {
	tval |= (1 << chky);		/* set bit **/
    }
    bttb[kpos] = chky;			/* write to bit swapping table **/
    kpos = ((kpos + 1) & 0x7);		/* next key read position **/
 }
 int tpos;				/* translate table position **/
 for(tpos = 0; tpos < 256; ++tpos) {	
    u_int8_t tval = tpos;		/* translate table index **/
    u_int8_t tres = 0;			/* translate value **/
    for(bpos = 0; bpos < 8; ++bpos) {		
	if((tval & (1 << bttb[bpos]))) {	/* if bit seted in inde **/
	    tres |= (1 << bpos);		/* set bit in translated value  **/
	}
    }
    *(ttab + tpos) = tres;		/* write to result value **/
 }
#if 0
 ucrp_printf(UMSG_CRLF);
 ucrp_printf(UMSG_BTTB);
 for(tpos = 0; tpos < 8; ++tpos) {
    ucrp_printf(UMSG_02X, (u_char) bttb[tpos]);
    ucrp_printf(UMSG_SP);
 }
 ucrp_printf(UMSG_CRLF);
 for(tpos = 0; tpos < 256; ++tpos) {
    ucrp_printf(UMSG_02X, (u_char) *(ttab + tpos));
    ucrp_printf(UMSG_SP);
 }
#endif 							/* 1 **/
 if(en == 1) {
    return(1);
 }
 u_int8_t rtab[256];			/* reverse table **/
 for(tpos = 0; tpos < 256; ++tpos) {	/* buffer position **/
    u_int8_t chtb;			/* readed character **/
    chtb = (ttab[tpos] & 0xFF);		/* read character from translate table **/
    rtab[chtb] = tpos;			/* write reverced value **/
 }
 for(tpos = 0; tpos < 256; ++tpos) {
    *(ttab + tpos) = rtab[tpos];
 }
#if 0
 ucrp_printf(UMSG_CRLF);
 ucrp_printf(UMSG_RTAB);
 for(tpos = 0; tpos < 256; ++tpos) {
    ucrp_printf(UMSG_02X, (u_char) *(ttab + tpos));
    ucrp_printf(UMSG_SP);
 }
#endif 							/* 1 **/
 return(1);
}
int ucrp_bts_en(u_char * dst, u_char * src, int len) /* bit swapping coding **/
{
 if(					/* check source and destination pointer **/	
    (dst == 0) || 
    (src == 0)
 ) {
    return(0);
 }
 u_int8_t ttab[256];			/* translate table **/
 if(ucrp_bts_init(ttab, 1, 1) < 0) {	/* init translate table **/
    return(0);
 }
 int bpos;				/* buffer position **/
 for(bpos = 0; bpos < len; ++bpos) {	
    u_int8_t chtb;			/* readed character **/
    chtb = *(src + bpos);		/* read character from source buffer **/
    chtb = ttab[(chtb & 0xFF)];		/* translate character **/
    *(dst + bpos) = chtb;		/* write character to destination buffer **/
 }
 return(len);
}
int ucrp_bts_de(u_char * dst, u_char * src, int len) /* bit swapping decoding **/
{
 if(					/* check source and destination pointer **/
    (dst == 0) || 
    (src == 0)
 ) {
    return(0);
 }
 u_int8_t ttab[256];			/* translate table **/
 if(ucrp_bts_init(ttab, 0, 1) < 0) {	/* init translate table **/
    return(0);
 }
 int bpos;				/* buffer position **/
 for(bpos = 0; bpos < len; ++bpos) {
    u_int8_t chtb;			/* read character **/
    chtb = *(src + bpos);		/* read character from source buffer **/
    chtb = ttab[(chtb & 0xFF)];		/* translate character **/
    *(dst + bpos) = chtb;		/* write character to destination buffer **/
 }
 return(len);				/* **/
}
#ifdef __UCRP_TAB_ENABLE__
int ucrp_tab_en(u_char * dst, u_char * src, int len) /* table encrypt **/
{
 u_char tab[256];			/* read primary table **/
 u_int32_t dpos;
 if(ucrp_comp_version() >= 118) {
    dpos = (
	(ucrp_key_val(ucrp_tab_base,UCRP_TAB_LEN,-1) + ucrp_passwd[4]) & 0xFF
    );
 } else {
    dpos = 0;
 } 
 int pos;				/* **/
 for(pos = 0; pos < 256; ++pos) {
    tab[dpos] = ucrp_key_val(ucrp_table_base + pos * 8, 8, -1);
    dpos += 1;
    if(dpos >= 256) {
	dpos = 0;
    }
 }
 u_int8_t btab[256];			/* translate table **/
 if(ucrp_bts_init(btab, 1, 0) < 0) {	/* init translate table **/
    return(0);
 }
 u_char xor_comp;
 u_char add_comp;
 u_char rot_comp;
 u_char bts_comp;
 u_char xor_val;			/* xor value **/
 u_char add_val;			/* add value **/
 u_char rot_val;			/* rotate value **/
 xor_comp = ucrp_comp(ucrp_xor_comp,-1);
 add_comp = ucrp_comp(ucrp_add_comp,-1);
 rot_comp = ucrp_comp(ucrp_rot_comp,-1);
 bts_comp = ucrp_comp(ucrp_bts_comp,-1);
/* read xor,add, rot values **/
 xor_val  = ucrp_key_val(UCRP_XOR_BASE,UCRP_XOR_LEN,-1);
 add_val  = ucrp_key_val(UCRP_ADD_BASE,UCRP_ADD_LEN,-1);
 rot_val  = (ucrp_key_val(UCRP_ROT_BASE,UCRP_ROT_LEN,-1) & 7);
 for(pos = 0; pos < len; ++pos) {	/* copy to destination buffer**/
    *(dst + pos) = *(src + pos);
 }
 int hlen;				/* **/ 
 if(ucrp_comp_version() >= 118) {
    hlen = 256;				/* **/ 
 } else {
    hlen = 64;
 }
 int hnum;				/* **/ 
 for(hnum = 0; hnum < hlen; ++hnum) {
    u_char hch;    
#ifdef __UCRP_TAB_DEBUG__
    {
	ucrp_printf(UMSG_TAB_EN, hnum);
        int tpos;
	for(tpos = 0; tpos < 256; ++tpos) {
	    ucrp_printf(UMSG_SP);
	    ucrp_printf(UMSG_02X, tab[tpos]);
	}	
	ucrp_printf(UMSG_CRNL);
    }
#endif 						/* __UCRP_TAB_DEBUG__ **/
    for(pos = 0; pos < len; ++pos) {
	u_char ch;
	ch = *(dst + pos);
	ch = tab[ch];
	*(dst + pos) = ch;
    }
    if(ucrp_comp_version() >= 118) {
	hch = (ucrp_passwd[hnum] >> 4);
    } else {
	hch = ucrp_passwd[hnum];
    }
    if(
	(ucrp_comp_version() < 118) ||
	(xor_comp > 0) 
    ) {
	if((hch & 1) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		tab[tpos] = (tab[tpos] ^ xor_val);
	    }
	}
    }
    if(
	(ucrp_comp_version() < 118) ||
	(add_comp > 0) 
    ) {
	if((hch & 2) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		tab[tpos] = tab[tpos] + add_val;
	    }
	}
    }
    if(
	(ucrp_comp_version() < 118) ||
	(rot_comp > 0) 
    ) {
	if((hch & 4) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		if(ucrp_comp_version() >= 118) {
		    tab[tpos] = (tab[tpos] << rot_val) | (tab[tpos] >> (8 - rot_val));
		} else {
		    tab[tpos] = (tab[tpos] >> rot_val) | (tab[tpos] << (8 - rot_val));
		}
	    }
	}
    }
    if(
	(ucrp_comp_version() >= 118) &&
	(bts_comp > 0) 
    ) {
	if((hch & 8) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		tab[tpos] = btab[tab[tpos]];
	    }
	}
    }
#ifdef __UCRP_TAB_DEBUG__
    {
	ucrp_printf(UMSG_TAB_EN, hnum);
	int tpos;
        for(tpos = 0; tpos < 256; ++tpos) {
	    ucrp_printf(UMSG_SP);
	    ucrp_printf(UMSG_02X, tab[tpos]);
	}	
	ucrp_printf(UMSG_CRNL);
    }
#endif 						/* __UCRP_TAB_DEBUG__ **/
    for(pos = 0; pos < len; ++pos) {
	u_char ch;
	ch = *(dst + pos);
	ch = tab[ch];
	*(dst + pos) = ch;
    }
    if(ucrp_comp_version() >= 118) {
	hch = ucrp_passwd[hnum];
    } else {
	hch = (ucrp_passwd[hnum] >> 4);
    }
    if(
	(ucrp_comp_version() < 118) ||
	(xor_comp > 0) 
    ) {
	if((hch & 1) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		tab[tpos] = (tab[tpos] ^ xor_val);
	    }
	}
    }
    if(
	(ucrp_comp_version() < 118) ||
	(add_comp > 0) 
    ) {
	if((hch & 2) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		tab[tpos] = tab[tpos] + add_val;
	    }
	}
    }
    if(
	(ucrp_comp_version() < 118) ||
	(rot_comp > 0) 
    ) {
	if((hch & 4) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		if(ucrp_comp_version() >= 118) {
	    	    tab[tpos] = (tab[tpos] << rot_val) | (tab[tpos] >> (8 - rot_val));
		} else {
	    	    tab[tpos] = (tab[tpos] >> rot_val) | (tab[tpos] << (8 - rot_val));
		}
	    }
	}
    }
    if(
	(ucrp_comp_version() >= 118) &&
	(bts_comp > 0) 
    ) {
	if((hch & 8) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		tab[tpos] = btab[tab[tpos]];
	    }
	}
    }
 }
#ifdef __UCRP_TAB_DEBUG__
    {
	ucrp_printf(UMSG_TAB_EN_END);
	int tpos;
	for(tpos = 0; tpos < 256; ++tpos) {
	    ucrp_printf(UMSG_SP);
	    ucrp_printf(UMSG_02X, tab[tpos]);
	}	
	ucrp_printf(UMSG_CRNL);
    }
#endif 						/* __UCRP_TAB_DEBUG__ **/
 return(len);
}
#endif 							/* __UCRP_TAB_ENABLE__ **/
#ifdef __UCRP_TAB_ENABLE__
int ucrp_tab_de(u_char * dst, u_char * src, int len)	/* table decrypt **/
{
 u_char tab[256];			/* read primary table **/	
 u_int32_t dpos;
 if(ucrp_comp_version() >= 118) {
    dpos = (
	(ucrp_key_val(ucrp_tab_base,UCRP_TAB_LEN,-1) + ucrp_passwd[4]) & 0xFF
    );
 } else {
    dpos = 0;
 } 
 int pos;				/* **/
 for(pos = 0; pos < 256; ++pos) {
    tab[dpos] = ucrp_key_val(ucrp_table_base + pos * 8, 8, -1);
    dpos += 1;
    if(dpos >= 256) {
	dpos = 0;
    }
 }
 u_int8_t btab[256];			/* translate table **/
 if(ucrp_bts_init(btab, 1, 0) < 0) {	/* init translate table **/
    return(0);
 }
 u_char xor_comp;
 u_char add_comp;
 u_char rot_comp;
 u_char bts_comp;
/* read xor, add, rot values **/
 u_char xor_val;
 u_char add_val;
 u_char rot_val;
 xor_comp = ucrp_comp(ucrp_xor_comp,-1);
 add_comp = ucrp_comp(ucrp_add_comp,-1);
 rot_comp = ucrp_comp(ucrp_rot_comp,-1);
 bts_comp = ucrp_comp(ucrp_bts_comp,-1);
 xor_val = ucrp_key_val(UCRP_XOR_BASE,UCRP_XOR_LEN,-1);
 add_val = ucrp_key_val(UCRP_ADD_BASE,UCRP_ADD_LEN,-1);
 rot_val = (ucrp_key_val(UCRP_ROT_BASE,UCRP_ROT_LEN,-1) & 7);
 for(pos = 0; pos < len; ++pos) {	/* copy to destination buffer **/
    *(dst + pos) = *(src + pos);
 }
 int hlen;				/* **/ 
 if(ucrp_comp_version() >= 118) {
    hlen = 256;				/* **/ 
 } else {
    hlen = 64;
 }
 int hnum;				/* **/ 
 for(hnum = 0; hnum < hlen; ++hnum) {
    u_char hch;    
    if(ucrp_comp_version() >= 118) {
	hch = (ucrp_passwd[hnum] >> 4);
    } else {
	hch = ucrp_passwd[hnum];
    }
    if(
	(ucrp_comp_version() < 118) ||
	(xor_comp > 0) 
    ) {
	if((hch & 1) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		tab[tpos] = (tab[tpos] ^ xor_val);
	    }
	}
    }
    if(
	(ucrp_comp_version() < 118) ||
	(add_comp > 0) 
    ) {
	if((hch & 2) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		tab[tpos] = tab[tpos] + add_val;
	    }
	}
    }
    if(
	(ucrp_comp_version() < 118) ||
	(rot_comp > 0) 
    ) {
	if((hch & 4) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		if(ucrp_comp_version() >= 118) {
		    tab[tpos] = (tab[tpos] << rot_val) | (tab[tpos] >> (8 - rot_val));
		} else {
	    	    tab[tpos] = (tab[tpos] >> rot_val) | (tab[tpos] << (8 - rot_val));
		}
	    }
	}
    }
    if(
	(ucrp_comp_version() >= 118) &&
	(bts_comp > 0) 
    ) {
	if((hch & 8) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		tab[tpos] = btab[tab[tpos]];
	    }
	}
    }
    if(ucrp_comp_version() >= 118) {
	hch = ucrp_passwd[hnum];
    } else {
	hch = (ucrp_passwd[hnum] >> 4);
    }
    if(
	(ucrp_comp_version() < 118) ||
	(xor_comp > 0) 
    ) {
	if((hch & 1) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		tab[tpos] = (tab[tpos] ^ xor_val);
	    }
	}
    }
    if(
	(ucrp_comp_version() < 118) ||
	(add_comp > 0) 
    ) {
	if((hch & 2) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		tab[tpos] = tab[tpos] + add_val;
    	    }
	}
    }
    if(
	(ucrp_comp_version() < 118) ||
	(rot_comp > 0) 
    ) {
	if((hch & 4) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		if(ucrp_comp_version() >= 118) {
		    tab[tpos] = (tab[tpos] << rot_val) | (tab[tpos] >> (8 - rot_val));
		} else {
		    tab[tpos] = (tab[tpos] >> rot_val) | (tab[tpos] << (8 - rot_val));
		}
	    }
	}
    }
    if(
	(ucrp_comp_version() >= 118) &&
	(bts_comp > 0) 
    ) {
	if((hch & 8) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		tab[tpos] = btab[tab[tpos]];
	    }
	}
    }
 }
#ifdef __UCRP_TAB_DEBUG__
    {
	ucrp_printf(UMSG_TAB_DE_END);
        int tpos;
	for(tpos = 0; tpos < 256; ++tpos) {
	    ucrp_printf(UMSG_SP);
	    ucrp_printf(UMSG_02X, tab[tpos]);
	}	
	ucrp_printf(UMSG_CRNL);
    }
#endif 						/* __UCRP_TAB_DEBUG__ **/
 if(ucrp_bts_init(btab, 0, 0) < 0) {	/* init translate table **/
    return(0);
 }
 for(hnum = hlen - 1; hnum >= 0; --hnum) {
    u_char hch;    
    if(ucrp_comp_version() >= 118) {
	hch = ucrp_passwd[hnum];
    } else {
	hch = (ucrp_passwd[hnum] >> 4);
    }
    if(
	(ucrp_comp_version() >= 118) &&
	(bts_comp > 0) 
    ) {
	if((hch & 8) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		tab[tpos] = btab[tab[tpos]];
	    }
	}
    }
    if(
	(ucrp_comp_version() < 118) ||
	(rot_comp > 0) 
    ) {
	if((hch & 4) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		if(ucrp_comp_version() >= 118) {
		    tab[tpos] = (tab[tpos] >> rot_val) | (tab[tpos] << (8 - rot_val));
		} else {
		    tab[tpos] = (tab[tpos] << rot_val) | (tab[tpos] >> (8 - rot_val));
		}
	    }
	}
    }
    if(
	(ucrp_comp_version() < 118) ||
	(add_comp > 0) 
    ) {
	if((hch & 2) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		tab[tpos] = tab[tpos] - add_val;
	    }
	}
    }
    if(
	(ucrp_comp_version() < 118) ||
	(xor_comp > 0) 
    ) {
	if((hch & 1) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		tab[tpos] = (tab[tpos] ^ xor_val);
	    }
	}
    }
#ifdef __UCRP_TAB_DEBUG__
    {
	ucrp_printf(UMSG_TAB_DE, hnum);
	int tpos;
	for(tpos = 0; tpos < 256; ++tpos) {
	    ucrp_printf(UMSG_SP);
	    ucrp_printf(UMSG_02X,tab[tpos]);
	}	
        ucrp_printf(UMSG_CRNL);
    }
#endif 						/* __UCRP_TAB_DEBUG__ **/
    u_char rev[256];
    for(pos = 0; pos < 256; ++pos) {
	u_char ch;
	ch = tab[pos];
        rev[ch] = pos;
    }
    for(pos = 0; pos < len; ++pos) {
	u_char ch;
	ch = *(dst + pos);
	ch = rev[ch];
	*(dst + pos) = ch;
    }
    if(ucrp_comp_version() >= 118) {
	hch = (ucrp_passwd[hnum] >> 4);
    } else {
	hch = ucrp_passwd[hnum];
    }
    if(
	(ucrp_comp_version() >= 118) &&
	(bts_comp > 0) 
    ) {
	if((hch & 8) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		tab[tpos] = btab[tab[tpos]];
	    }
	}
    }
    if(
	(ucrp_comp_version() < 118) ||
	(rot_comp > 0) 
    ) {
	if((hch & 4) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		if(ucrp_comp_version() >= 118) {
		    tab[tpos] = (tab[tpos] >> rot_val) | (tab[tpos] << (8 - rot_val));
		} else {
		    tab[tpos] = (tab[tpos] << rot_val) | (tab[tpos] >> (8 - rot_val));
		}
	    }
	}
    }
    if(
	(ucrp_comp_version() < 118) ||
	(add_comp > 0) 
    ) {
	if((hch & 2) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		tab[tpos] = tab[tpos] - add_val;
	    }
	}
    }
    if(
	(ucrp_comp_version() < 118) ||
	(xor_comp > 0) 
    ) {
	if((hch & 1) != 0) {
	    int tpos;
	    for(tpos = 0; tpos < 256; ++tpos) {
		tab[tpos] = (tab[tpos] ^ xor_val);
	    }
	}
    }
#ifdef __UCRP_TAB_DEBUG__
    {
	ucrp_printf(UMSG_TAB_DE, hnum);
        int tpos;
	for(tpos = 0; tpos < 256; ++tpos) {
	    ucrp_printf(UMSG_SP);
	    ucrp_printf(UMSG_02X, tab[tpos]);
	}	
	ucrp_printf(UMSG_CRNL);
    }
#endif 						/* __UCRP_TAB_DEBUG__ **/
    for(pos = 0; pos < 256; ++pos) {
	u_char ch;
	ch = tab[pos];
        rev[ch] = pos;
    }
    for(pos = 0; pos < len; ++pos) {
	u_char ch;
	ch = *(dst + pos);
	ch = rev[ch];
	*(dst + pos) = ch;
    }
 }
 return(len);
}
#endif 							/* __UCRP_TAB_ENABLE__ **/
/* **/
extern void nand_calculate_ecc (const u_char *dat, u_char *ecc_code);
extern int nand_correct_data (u_char *dat, u_char *read_ecc, u_char *calc_ecc);
#ifdef __UCRP_ECC_ENABLE__
//#define NO_ECC
int ucrp_ecc_en(u_char * dst, u_char * src, int len)
{
 if(
    (dst == 0) ||			/* destination buffer pointer check */
    (src == 0)				/* source buffer pointer check */
 ) {
    return(0);
 }
#ifdef NO_ECC
 memcpy(dst,src,len);
 return(len);
#else
 u_char ecc[3];				/* calculated ecc **/
 int dpos;				/* destination position **/
 int spos; 				/* source position **/
 dpos = 0;				/* start destination position **/
 for(spos = 0; spos < len;) {
    int cnt;
    if(spos + 256 <= len) {		/* full block **/
	nand_calculate_ecc(src + spos, ecc);	/* calculate ecc **/
	cnt = 256;			/* source block size **/
    } else {				/* partitional block **/
	cnt = len - spos;		/* block size **/
	u_char tmp[256];		/* temp buffer **/
	int pos;			/* buffer position **/
	for(pos = 0; pos < cnt; ++pos) {   /* copy to temp buffer **/
	    tmp[pos] = *(src + spos + pos);
	}
	while(pos < 256) {		/* fill tail **/
	    tmp[pos++] = 0xFF;
	}
	nand_calculate_ecc(tmp, ecc);	/* calculate ecc **/
    }
    int pos;				/* byte number **/
    for(pos = 0; pos < cnt; ++pos) {	/* copy data to destination buffer **/	
	*(dst + dpos) = *(src + spos);	/* copy byte **/
	 dpos += 1;			/* destination position **/
  	 spos += 1;			/* source position **/
    }
    *(dst + dpos++) = ecc[0];		/* copy ecc to destination buffer **/ /* ecc 0 **/
    *(dst + dpos++) = ecc[1];		/* ecc 1 **/
    *(dst + dpos++) = ecc[2];		/* ecc 2 **/
 }
 return(dpos);
#endif								/* NO_ECC **/
}
#endif 							/* __UCRP_ECC_ENABLE__ **/
#ifdef __UCRP_ECC_ENABLE__
int ucrp_ecc_de(u_char * dst, u_char * src, int len) /* ecc decode **/
{
 if(
    (dst == 0) ||			/* destination buffer pointer check */
    (src == 0)				/* source buffer pointer check */
 ) {
    return(0);
 }
#ifdef NO_ECC
 memcpy(dst,src,len);
 return(len);
#else
 u_char calc[3];
 u_char * ecc;				/* readded ecc **/
 int dpos;				/* destination position **/
 int spos; 				/* source position **/
 dpos = 0;				/* start destination position  **/
 for(spos = 0; spos < len;) {
    int cnt;
    if(spos + 256 + 3 <= len) {		/* full block **/
	ecc = src + spos + 256;		/* readed ecc **/
	nand_calculate_ecc(src + spos, calc);	/* claculate ecc **/	
	nand_correct_data(src + spos, ecc, calc); /* correct data **/
	cnt = 256;			/* bytes count **/	
	int pos;			/* byte offset **/
	for(pos = 0; pos < cnt; ++pos) {
	    *(dst + dpos) = *(src + spos);	/* copy byte **/
	     dpos += 1;			/* move destination position **/
	     spos += 1;			/* move source position **/
	}
	spos += 3;			/* skip ecc code **/
    } else {
	cnt = len - spos - 3;		/* bytes **/
	u_char tmp[256];		/* temp buffer **/
	int pos;			/* byte offset **/
	for(pos = 0; pos < cnt; ++pos) {	/* copy to temp buffer **/
	    tmp[pos] = *(src + spos + pos);
	}
	ecc = src + spos + pos;		/* readed ecc **/
	while(pos < 256) {		/* fill tail **/
	    tmp[pos++] = 0xFF;
	}
	nand_calculate_ecc(tmp, calc);	/* calculate ecc **/
	nand_correct_data(tmp, ecc, calc); /* correct data **/
	for(pos = 0; pos < cnt; ++pos) { /* copy to destination buffer **/		
	    *(dst + dpos) = *(tmp + pos); /* copy bytes **/
	    dpos += 1;			/* next destinatiuon position **/
	    spos += 1;			/* next source position **/
	}
	spos += 3;			/* skip ecc **/
    }
 }
 return(dpos);				/* return total bytes **/
#endif 							/* NO_ECC **/
}
#endif 	
int ucrp_proc_clear(void)		/* clear disk space **/
{
 char * buf;				/* buffer **/
 u_int32_t bufsize;			/* buffer size **/
 bufsize = 0x10000;			/* 64K **/
 buf = (char *)malloc(bufsize);		/* allocate buffer **/
 if(buf == 0) {				/* null buffer pointer **/
    ucrp_error(				/* buffer allocation error **/
	"Buffer allocation error\r\n");
    return(0);
 }
 int rvl;				/* return value **/
 rvl = 1;				/* start return code **/
 {
    struct passwd * pwd;		/* passwd struct **/
    uid_t uid;				/* user id **/
    char * hname;			/* home directory **/
    char * sstr;			/* tail home name separator **/
    uid = geteuid();			/* get effective user id **/
    pwd = getpwuid(uid);		/* home directory **/
    hname = "root";			/* start home directory **/
    sstr = "/";				/* separator string **/
    if(pwd != 0) {			/* not null passwd struct **/
	if(pwd->pw_name != 0) {		/* not null user name**/
	    if(pwd->pw_dir != 0) {	/* not null home directory **/
		hname = pwd->pw_dir;	/* reset home directory **/
		if(*hname == '/') {	
		    hname += 1;		/* sckip start separator **/
		}
		int hlen;		/* home directory length **/
		hlen = strlen(hname);	/* calculate home directory length **/
		if(hlen > 0) {
		    if(*(hname + (hlen - 1)) == '/') {
			sstr = "";	/* clear end separator **/
		    }
		}
	    }
	}
    }
    char *fmt[2] = { 			/* files name formats **/
	"/%s%s.bash_history",		/* bash history **/
	"/%s%s.mc/history"		/* mc history **/
    };
    int nnum;				/* name number **/
    for(nnum = 0; nnum < 2; ++nnum) {
	snprintf(			/* form file name **/
	    buf,			/* file name buffer **/
	    bufsize,			/* buffer size **/
	    fmt[nnum],			/* format **/
	    hname,			/* home directory **/ 
	    sstr			/* separator string **/
	);
	if(ucrp_file("f",buf) > 0) {
	/* file exist **/
	    ucrp_printf("unlink %15s\r\n",buf);
	    if(ucrp_clear(buf) <= 0) { /* clear file **/
		rvl = 0;		/* clear file error **/
	    }
	    if(ucrp_unlink(buf) <= 0) { /* unlink file **/
		rvl = 0;		/* unlink file error **/
	    }
	} else {
	/* print absent message **/
	    ucrp_printf("absent %15s\r\n", buf);
	}
    }
 }
 int fnum;				/* file number **/
 fnum = 0;				/* start file number **/
 while(1) {				/* write files **/
    snprintf(				/* form file name **/
	buf, 				/* file name buffer **/
	bufsize,			/* file name buffer size **/
	"ucrp_clear.%03d", 		/* format **/
	fnum				/* file number **/
    );
    int fdes;				/* file descriptor **/
    fdes = open(buf, (O_RDWR|O_CREAT)); /* open file **/
    if(fdes < 0) {
	rvl = 0;			/* clear return code **/
	ucrp_error("File open error [%s]\r\n",buf);
	break;
    }
    ucrp_printf("\r\n%15s%15d",buf, 0);	/* file position **/
    u_int32_t pos; 			/* buffer position **/
    pos = 0;				/* dile position **/
    pos = 0;				/* dile position **/
    while(pos < bufsize) {
	if(ucrp_clear_mode == UCRP_CLEAR_CLR) {
	    *(buf + pos) = 0;
	} else
	if(ucrp_clear_mode == UCRP_CLEAR_SET) {
	    *(buf + pos) = 0xFF;
	} else
	if(ucrp_clear_mode == UCRP_CLEAR_INC) {
	    *(buf + pos) = pos;
	} else {
	    *(buf + pos) = pos;
	}
	pos += 1;
    }	
    while(pos < 1024 * (1024 * 1024)) {	/* write write buffer **/
	int wcnt;    			/* writed bytes **/
	wcnt = write(fdes, buf, bufsize); /* write buffer **/	
	if(wcnt < 0) {			
	    break;			/* write buffer error **/
	}
	pos += wcnt;			/* calculate file position **/
	if((pos % (1024 * 128)) == 0) {	/* print file position **/
	    int bsn;			/* backspace number  **/	
	    for(bsn = 0; bsn < 15; ++bsn) { /* clear file size **/
		ucrp_printf("\b");
	    }
	    ucrp_printf("%15d", pos);
	}
	if(wcnt != (int)bufsize) {
	    break;			/* not enoth disk space **/
	}
    }
    {
	int bsn;
	for(bsn = 0; bsn < 15; ++bsn) {
	    ucrp_printf("\b");
	}
	ucrp_printf("%15d\r\n", pos);
    }
    if(fsync(fdes) < 0) {		/* file sync **/
	rvl = 0;			/* file sync error **/
    }
    if(fdes != 0) {
	if(close(fdes) < 0) {		/* close file **/
	    rvl = 0;			/* close file error **/
	}
	fdes = 0;			/* clear file descriptor **/
    }
    if(pos < 1024 * (1024 * 1024)) {
	break;				/* files writed **/
    }
    fnum += 1;				/* next file number **/
 }
 while(fnum >= 0) {			/* remove files **/
    snprintf(				/* form file name **/
	buf,				/* file name buffer **/
	bufsize,			/* file name buffer size **/
	"ucrp_clear.%03d",		/* format **/
	fnum				/* file number **/
    );
    if(ucrp_file("f",buf) > 0) {
	ucrp_printf("unlink %15s\r\n",buf); /* print unlink message **/ 
	if(unlink(buf) < 0) {		/* unlink file **/
	    rvl = 0;			/* clear return code **/
	}
    }
    fnum -= 1;				/* unlink previous file **/
 }
 if(buf != 0) {
    free(buf);				/* free write buffer **/
    buf = 0;				/* cler write buffer **/
 }
 return(rvl);
}




#include <sched.h>
static int ucrp_argc = 0;		/* user argument count **/
static char **ucrp_args = 0;		/* argument string **/
static char ucrp_stack[1024 * 128];	/* program stack **/
static char * ucrp_sig_name(siginfo_t * sinfo, char * sname, int ssize) /* print signal name **/
{
 static struct {
    int num;				/* signal number **/
    char * name;			/* signal name **/
 } sginfo[] = {
    { SIGHUP   , "1)SIGHUP - Hangup" },
    { SIGINT   , "2)SIGINT - Interrupt" },
    { SIGQUIT  , "3)SIGQUIT - Quit" },
    { SIGILL   , "4)SIGILL - Illegal instruction" },
    { SIGTRAP  , "5)SIGTRAP - trap" },
    { SIGABRT  , "6)SIGABRT - abort" },
    { SIGBUS   , "7)SIGBUS - buss error" },
    { SIGFPE   , "8)SIGFPE - floating point exeption" },
/* not processed signal **/ 
    { SIGKILL  , "9)SIGKILL - kill" },
    { SIGUSR1  , "10)SIGUSR1 - user signal 1" },
    { SIGSEGV  , "11)SIGSEGV - segmetation violation" },
    { SIGUSR2  , "12)SIGUSR2 - user signal 2" },
    { SIGPIPE  , "13)SIGPIPE - pipe" },
    { SIGALRM  , "14)SIGALRM - alarm" },
    { SIGTERM  , "15)SIGTERM - term" },
    { SIGSTKFLT, "16)SIGSTKFLT - stack failt" },
/* 17-31 **/
    { SIGCHLD  , "17)SIGCHLD - children" },
    { SIGCONT  , "18)SIGCONT - continue" },
    { SIGSTOP  , "19)SIGSTOP - stop" },
    { SIGTSTP  , "20)SIGTSTP - terminal stop, terminal command" },
    { SIGTTIN  , "21)SIGTTIN - terminal read" },
    { SIGTTOU  , "22)SIGTTOU - terminal weite" },
    { SIGURG   , "23)SIGURG - urg" },	/*  **/
    { SIGXCPU  , "24)SIGXCPU - cpu limit exceeded" },
/* **/
    { SIGXFSZ  , "25)SIGXFSZ - file size exceeded" },
    { SIGVTALRM, "26)SIGVTALRM - virtual alarm clock" },
    { SIGPROF  , "27)SIGPROF - profilling alarm clock" },
    { SIGWINCH , "28)SIGWINCH - window size changed" },
    { SIGIO    , "29)SIGIO - I/O" },
    { SIGPWR   , "30)SIGPWR - power failure restart" },
    { SIGSYS   , "31)SIGSYS - bad system call" }
 };
 char * sptr;				/* signal name pointer **/
 if(sinfo->si_signo >= 1 && sinfo->si_signo <= 31) {
    if(sginfo[sinfo->si_signo-1].num == sinfo->si_signo) {
	sptr = sginfo[sinfo->si_signo-1].name; /* print signal description **/
    } else {
	return(0);
    }
 } else {
    return(0);
 }
 if(sname == 0 || ssize <= 1) {
    return(sptr);
 }
 snprintf(
    sname,				/* signal name buffer **/
    ssize,				/* buffer size **/
    "%s",				/* format **/
    sptr
 );    
 return(sname);				/* return buffer pointer **/
} 
static int ucrp_sig_info(siginfo_t * sinfo, void * sptr) /* print signal information **/
{
 if((sinfo == 0) || (sptr == 0)) { /* pointers null check -> print error message and exit **/
    ucrp_error("Bad signal call sinfo=0x%x , sptr=0x%x\n",
	sinfo	,		/* signal pointer **/
	sptr			/* **/
    );
    return(0);
 }
 char sname[32];
 if(ucrp_sig_name(sinfo,sname,sizeof(sname)) != 0) {
    ucrp_printf("%s\n", sname);	/* print signal description **/
 }
 ucrp_printf(			/* print signal description **/
    "Signal num {%d}, errno {%d}, code {%d}\n"
    "pid {%d}, uid {%d}, status {%d}, utime {%d}, stime {%d}\n"
    "value {%d}, int {%d}, ptr {%d}, addr {%d}, band {%d}, fd {%d}\n"
    "sptr {%x}\n",
    sinfo->si_signo	,	/* 1 signal number **/
    sinfo->si_errno	,	/* 1 errno code **/
    sinfo->si_code 	,	/* 1 code **/
    sinfo->si_pid 	,	/* 2 process id **/
    sinfo->si_uid	,	/* 2 user id **/
    sinfo->si_status	,	/* 2 **/
    sinfo->si_utime	,	/* 2 user time **/
    sinfo->si_stime	,	/* 2 system time **/
    sinfo->si_value	,	/* 3 signal vale **/
    sinfo->si_int 	,	/* 3 int **/
    sinfo->si_ptr	,	/* 3 ptr **/
    sinfo->si_addr	,	/* 3 addr **/
    sinfo->si_band	,	/* 3 **/
    sinfo->si_fd	,	/* 3 file descriptor **/
    sptr			/* 4 **/
 );
 return(1);	
}
int ucrp_sig_log(siginfo_t * sinfo, void * sptr) /* log signal **/
{
 if(sinfo == 0) {			/* signal description null pointer **/	
    return(0);
 } 
 char * lname = "/var/log/ucrpsdump";	/* log file name **/
 FILE * flog;				/* log file **/
 int rvl;				/* return code **/
 rvl = 1;				/* start return code **/
 flog = fopen(lname, "r+b");		/* open log file  **/
 if(flog == 0) {
    flog = fopen(lname, "w");		/* open log file  **/
    if(flog == 0) {			/* open file in write mode error **/
	return(0);
    }
 } else {
    if(fseek(flog, 0, SEEK_END) < 0) { /* file opened in read mode -> set to file end **/
	rvl = 0;
    }
 }
 char buf[32];    			/* signal name buffer **/
 if(ucrp_sig_name(sinfo,buf,sizeof(buf)) <= 0) {
    fprintf(flog, "%d\n", sinfo->si_signo);
 } else {
    fprintf(flog, "%s\n", buf);
 }
 if(fclose(flog) < 0) {
    rvl = 0;
 }
 if(rvl == 0) {
    return(0);
 }
 return(1);
}
volatile int kill_link = 0;		/* terminate signal received **/
static void ucrp_sig_term_ext(int sig, siginfo_t * sinfo, void * sptr)
{
 if(kill_link == 0) {
    kill_link = 1;			/* received signal **/
    ucrp_sig_info(sinfo,sptr);		/* print signal description **/
    ucrp_sig_log(sinfo,sptr);		/* log signal **/
 } else {
    kill_link += 1;			/* received signal **/
 }
}
volatile u_int8_t ucrp_debug_alrm = 1;	/* print debug note in alarm handler **/
volatile u_int8_t ucrp_sig_alrm = 0;	/* > 1, exit from select loop **/
static void ucrp_sig_alarm_ext(int sig, siginfo_t * sinfo, void * sptr)
{
 if(ucrp_sig_alrm == 0) {
    if(ucrp_debug_alrm > 0) {		/* debug alarm signals **/
	ucrp_sig_info(sinfo,sptr);	/* print signal description **/
	ucrp_sig_log(sinfo,sptr);	/* log signal **/
    }
 }
 ucrp_sig_alrm += 1;				/* > 1, exit from select loop **/
}
volatile int ucrp_sig_pipe = 0;		/* sigpipe processing function  **/
static void ucrp_sig_pipe_ext(int sig, siginfo_t * sinfo, void * sptr)
{
 ucrp_sig_info(sinfo,sptr);		/* print signal description **/
 ucrp_sig_log(sinfo,sptr);		/* log signal **/
 ucrp_sig_pipe = 1;
}
volatile int ucrp_sig_chld = 0;		/* SIGTERM/SIGINT **/
static void ucrp_sig_chld_ext(int sig, siginfo_t * sinfo, void * sptr)
{
 ucrp_sig_info(sinfo,sptr);		/* print signal description **/
 ucrp_sig_log(sinfo,sptr);		/* log signal **/
 ucrp_sig_chld = 1;
}
static void ucrp_bad_signal_ext(int sig, siginfo_t * sinfo, void * sptr) /* bad signal received **/
{
 ucrp_sig_info(sinfo,sptr);		/* print signal description **/
 ucrp_sig_log(sinfo,sptr);		/* log signal **/
 exit(1);		
}
int ucrp_signal(int snum, void (*actn)(int, siginfo_t *, void *)) 
{
 if(
    (snum == SIGKILL) || 		/* unconditionall kill **/
    (snum == SIGSTOP)			/* stop process **/
 ) {
    return(0);				/* not processed signals **/
 } 
 struct sigaction sact;			/* signal action **/
 sact.sa_handler = 0;			/* signal handler **/
 sact.sa_sigaction = actn;		/* signal extended handler **/
 sact.sa_restorer = 0;			/* restore function **/
 sact.sa_flags = (			/* signal process settings **/
    SA_SIGINFO | 			/* signal handler requiry 3 arguments **/
    SA_RESTART | 			/* BSD compatible **/
    SA_NOMASK				/* not mask this signal **/
 );
 if(sigemptyset(&sact.sa_mask) < 0) {	/* clear blocked signal mask **/
    ucrp_error(				/* clear mask error **/
	"Clear nonblocked signals mask error\n");
    return(0);
 }
 if(					/* blocked signals mask **/ 
    (sigaddset(&sact.sa_mask, SIGHUP)  < 0) || 	/* SIGHUP **/
    (sigaddset(&sact.sa_mask, SIGINT)  < 0) ||	/* INT **/
    (sigaddset(&sact.sa_mask, SIGTERM) < 0) ||	/* TERM **/
    (sigaddset(&sact.sa_mask, SIGCHLD) < 0) ||	/* CHLD **/
    (sigaddset(&sact.sa_mask, SIGUSR2) < 0)   	/* USR2 **/
 ) {	
    ucrp_error(				/* add signal in mask error **/
	"Set nonblocked signals mask %x error\n", sact.sa_mask);
    return(0);
 }
 if(sigaction(snum, &sact, 0) < 0) {	/* set signal handler **/
    ucrp_error("Set signal %d handler error\n");
    return(0);
 }
 return(1);
}
int main(int argc, char * args[])
{
 extern int ucrp_main(void *);		/* clone function **/
 int status;
 int pid;
 int rvl;
 rvl = 1; 
 ucrp_argc = argc;			/* save argument count **/
 ucrp_args = args;			/* save argument string **/
 pid = clone(				/* create clone with new stack **/
    ucrp_main, ucrp_stack + sizeof(ucrp_stack) -4 , CLONE_VFORK | CLONE_DETACHED, (void *) UMSG_EMP);
 status = 0;				/* clear program exit status **/
 wait(& status);			/* wait program termination **/
 if(status == 0) {			
    rvl = 0;				/* clear program return code **/
 }
 exit(rvl);				/* return **/
}  
int ucrp_main(void *)			/* clone function **/
{
 {
    ucrp_path_size = _POSIX_NAME_MAX;	/* file name buffer size **/
    if(ucrp_path_size < _POSIX_PATH_MAX) {	
	ucrp_path_size = _POSIX_PATH_MAX;  /* reset to posix path size **/
    }
 }
 ucrp_init();
 int argn;
 if(ucrp_argc >= 1) {
    if( (*(ucrp_args[0] + 0) == '.') || (*(ucrp_args[0] + 1) == '/') ) {
	ucrp_local = 1;			/* local path **/
    } else {
	ucrp_local = 0;			/* not local path **/
    } 
 }
 u_int16_t ecmd = 0;
 int16_t argd;
 int rvl; 
 rvl = 1;
 int inum;				/* signal definition index **/	
 inum = 0;				/* start index **/		
 while(1) {
    struct {
	int snum;				/* signal number **/
	void (*sfun)(int, siginfo_t *, void *); /* signal function **/
    } sigdef[] = {
	{ SIGHUP   , ucrp_bad_signal_ext },	/* 1 - Hangup **/
	{ SIGINT   , ucrp_bad_signal_ext },	/* 2 - Interrupt **/
	{ SIGQUIT  , ucrp_bad_signal_ext },	/* 3 - Quit **/
	{ SIGILL   , ucrp_bad_signal_ext },	/* 4 - Illegal instruction **/

	{ SIGTRAP  , ucrp_bad_signal_ext },	/* 5 - trap **/
	{ SIGABRT  , ucrp_bad_signal_ext },	/* 6 - abort **/
	{ SIGBUS   , ucrp_bad_signal_ext },	/* 7 - buss error **/
	{ SIGFPE   , ucrp_bad_signal_ext },	/* 8 - floating point exeption **/
/* not processed signal **/ 
/*	{ SIGKILL  , ucrp_bad_signal_ext }, **/	/* 9 - kill **/
	{ SIGUSR1  , ucrp_bad_signal_ext },	/* 10 - user signal 1 **/
	{ SIGSEGV  , ucrp_bad_signal_ext },	/* 11 - segmetation violation **/
	{ SIGUSR2  , ucrp_bad_signal_ext },	/* 12 - user signal 2 **/
 
	{ SIGPIPE  , ucrp_sig_pipe_ext },	/* 13 - pipe **/
	{ SIGALRM  , ucrp_sig_alarm_ext },	/* 14 - alarm **/
	{ SIGTERM  , ucrp_sig_term_ext },	/* 15 - term **/
	{ SIGSTKFLT, ucrp_bad_signal_ext },	/* 16 - stack failt **/
/* 17-31 **/
	{ SIGCHLD  , ucrp_sig_chld_ext },	/* 17 - children **/
	{ SIGCONT  , ucrp_bad_signal_ext },	/* 18 - continue **/
/* not processed signal **/ 
/*	{ SIGSTOP  , ucrp_bad_signal_ext }, **/	/* 19 - stop **/
	{ SIGTSTP  , ucrp_bad_signal_ext },	/* 20 - terminal stop, terminal command **/
	{ SIGTTIN  , ucrp_bad_signal_ext },	/* 21 - terminal read **/
	{ SIGTTOU  , ucrp_bad_signal_ext },	/* 22 - terminal weite **/
	{ SIGURG   , ucrp_bad_signal_ext },	/* 23 - urg **/
	{ SIGXCPU  , ucrp_bad_signal_ext },	/* 24 - cpu limit exceeded **/
	{ SIGXFSZ  , ucrp_bad_signal_ext },	/* 25 - file size exceeded **/
	{ SIGVTALRM, ucrp_bad_signal_ext },	/* 26 - virtual alarm clock **/
	{ SIGPROF  , ucrp_bad_signal_ext },	/* 27 - profilling alarm clock **/
	{ SIGWINCH , ucrp_bad_signal_ext },	/* 28 - window size changed **/
	{ SIGIO    , ucrp_bad_signal_ext },	/* 29 - I/O **/
	{ SIGPWR   , ucrp_bad_signal_ext },	/* 30 - power failure restart **/
	{ SIGSYS   , ucrp_bad_signal_ext },	/* 31 - bad system call **/
/* 32 **/
/* 33 **/
/* 34 **/
	{ 0, 0 }
    };
    int snum;				/* signal number **/
    snum = sigdef[inum].snum;		/* read signal number **/
    if(snum == 0) {			/* signal array terminator **/
	{ break; }			/* terminate loop **/
    }
    if(ucrp_signal(snum,sigdef[inum].sfun) == 0) { /* set signal process function **/
        rvl = 0;			/* set signal error -> clear return code **/
    }
    inum += 1;				/* next index in signal description **/
 }
 {					/* 35 - 64 **/
    int snum;				/* signal number **/
    for(snum = SIGRTMIN; snum <	SIGRTMAX; ++snum) { /* set run time signals 35 - 64 **/
	if(ucrp_signal(snum,ucrp_bad_signal_ext) == 0) {
	    rvl = 0;			/* set signal error -> clear return code **/
	}
    }
 }
 for(argn = 1; argn < ucrp_argc; argn += argd) {
    argd = 1;				/* processed arguments **/
    if(strcmp(ucrp_args[argn], "--encrypt") == 0) {
	    ucrp_cmd |= UCRP_ENCRYPT;	/* encrypt **/
    } else
    if(strcmp(ucrp_args[argn], "--decrypt") == 0) {
	    ucrp_cmd |= UCRP_DECRYPT;	/* decrypt **/
    } else
    if(strcmp(ucrp_args[argn], "--clear") == 0) {
	    ucrp_cmd |= UCRP_CLEAR;	/* clear **/
	    if(argn + 1 < ucrp_argc) {
		if(strcmp(ucrp_args[argn+1], "inc") == 0) {
		    ucrp_clear_mode = UCRP_CLEAR_INC; 
		    argd = 2;
		} else
		if(strcmp(ucrp_args[argn+1],"0") == 0) {
		    ucrp_clear_mode = UCRP_CLEAR_CLR; 
		    argd = 2;
		} else
		if(strcmp(ucrp_args[argn+1],"1") == 0) {
		    ucrp_clear_mode = UCRP_CLEAR_SET; 
		    argd = 2;
		}
	    }
    } else
    if(strcmp(ucrp_args[argn], "--quiet") == 0) {
	ucrp_quiet = 1;			/* quiet mode **/
    } else
    if((strcmp(ucrp_args[argn], "--current-dir") == 0) ||
       (strcmp(ucrp_args[argn], "--change-dir") == 0)
    ) {
	if(argn + 1 >= ucrp_argc) {
	    ucrp_error("Option {%s} requery argument\r\n",
		ucrp_args[argn]
	    ); 
	    { break; }				/* **/
	} else {
	    snprintf(ucrp_cdir, sizeof(ucrp_cdir), 
	    UMSG_PS, ucrp_args[argn+1]);
	    argd = 2;
	}
    } else 
    if((strcmp(ucrp_args[argn], "--disable") == 0)) { /* disable algoritm **/
	if(argn + 1 >= ucrp_argc) {
	    ucrp_error("Option --disable requery argument\r\n"); 
	    { break; }				/* **/
	} else {
	    int argt;			/* temporary argument number **/
	    for(argt = argn + 1; argt < ucrp_argc; ++argt) {
		if(strcmp(ucrp_args[argt],"xor") == 0) {
		    ucrp_enable &= (~UCRP_XOR_ENABLE); /* disable byte xor **/
		} else
		if(strcmp(ucrp_args[argt],"add") == 0) { 
		    ucrp_enable &= (~UCRP_ADD_ENABLE); /* disable byte add **/
		} else
		if(strcmp(ucrp_args[argt],"rot") == 0) { 
		    ucrp_enable &= (~UCRP_ROT_ENABLE); /* disable byte rotate **/
		} else
		if(strcmp(ucrp_args[argt],"tab") == 0) { 
		    ucrp_enable &= (~UCRP_TAB_ENABLE); /* disable byte table **/
		} else
		if(strcmp(ucrp_args[argt],"ecc") == 0) { 
		    ucrp_enable &= (~UCRP_ECC_ENABLE); /* disable ecc **/
		} else
		if(strcmp(ucrp_args[argt],"bts") == 0) { 
		    ucrp_enable &= (~UCRP_BTS_ENABLE); /* disable bts **/
		} else
		if(strcmp(ucrp_args[argt],"bev") == 0) { 
		    ucrp_enable &= (~UCRP_BEV_ENABLE); /* disable bev **/
		} else
		if(strcmp(ucrp_args[argt],"xre") == 0) { 
		    ucrp_enable &= (~UCRP_XRE_ENABLE); /* disable xre algoritm **/
		} else
		if(strcmp(ucrp_args[argt],"std") == 0) { 
		    ucrp_enable = (0xFFFFFFFF & (~UCRP_XOR_ENABLE));
		} else {
		    break;
		}
	    }
	    argd = argt - argn;
	}
    } else 
    if((strcmp(ucrp_args[argn], "--enable") == 0)) { /* enable algoritm **/
	if(argn + 1 >= ucrp_argc) {
	    ucrp_error("Option --enable requery argument\r\n"); 
	    { break; }				/* **/
	} else {
	    int argt;			/* temporary argument number **/
	    for(argt = argn + 1; argt < ucrp_argc; ++argt) {
		if(strcmp(ucrp_args[argt],"xor") == 0) {
		    ucrp_enable |= UCRP_XOR_ENABLE; /* enable byte xor **/
		} else
		if(strcmp(ucrp_args[argt],"add") == 0) { 
		    ucrp_enable |= UCRP_ADD_ENABLE; /* enable byte add **/
		} else
		if(strcmp(ucrp_args[argt],"rot") == 0) { 
		    ucrp_enable |= UCRP_ROT_ENABLE; /* enable byte rotate **/
		} else
		if(strcmp(ucrp_args[argt],"tab") == 0) { 
		    ucrp_enable |= UCRP_TAB_ENABLE; /* enable byte table **/
		} else
		if(strcmp(ucrp_args[argt],"ecc") == 0) { 
		    ucrp_enable |= UCRP_ECC_ENABLE; /* enable ecc **/
		} else
		if(strcmp(ucrp_args[argt],"bts") == 0) { 
		    ucrp_enable |= UCRP_BTS_ENABLE; /* enable bts **/
		} else
		if(strcmp(ucrp_args[argt],"bev") == 0) { 
		    ucrp_enable |= UCRP_BEV_ENABLE; /* enable bev **/
		} else
		if(strcmp(ucrp_args[argt],"xre") == 0) { 
		    ucrp_enable |= UCRP_XRE_ENABLE; /* enable xre algoritm **/
		} else
		if(strcmp(ucrp_args[argt],"std") == 0) { 
		    ucrp_enable = (0xFFFFFFFF & (~UCRP_XOR_ENABLE));
		} else {
		    break;
		}
	    }
	    argd = argt - argn;
	}
    } else 
    if(strcmp(ucrp_args[argn], "--block-size") == 0) { /* block size **/
	if(argn + 1 >= ucrp_argc) {
	    ucrp_error("Option --block-size requery argument\r\n"); /* print error message **/
	    { break; }				/* exit **/
	} else {
	    int valu;				/* numeric argument **/
	    char * sstr;			/* text string **/
	    sstr = ucrp_args[argn+1];		/* read text string from command args **/
	    if(sstr != 0) {
		if((*sstr == 'x') || (*sstr == 'X')) { 
		    char * endptr;	/* read number from command ucrp_args **/
		    valu = strtoul(sstr + 1, &endptr, 16);
		} else 
		if(
		       (*sstr == '0') && 
		    ((*(sstr + 1) == 'x') || (*(sstr + 1) == 'X'))
		) {			/* read number from command ucrp_args **/
		    char * endptr;
		    valu = strtoul(sstr + 2, &endptr, 16);
		} else {
		    valu = atoi(sstr);	/* read number from command ucrp_args **/
		}
	    } else {
		valu = 256;		/* null pointer **/
	    }
	    if(valu < 256) {		/* set value to low limit **/
		valu = 256; 			
	    }
	    ucrp_src_buf_size = ((valu + 255) / 256) * 256; /* source buffer size **/
	    ucrp_dst_buf_size = 	/* destination buffer size **/
		ucrp_src_buf_size + (((ucrp_src_buf_size + 255) / 256) * 3);
	    argd = 2;
	}
    } else
    if(strcmp(ucrp_args[argn], "--passwd") == 0) {
	    if(argn + 1 >= ucrp_argc) {
		ucrp_error("Option --passwd requery argument\r\n"); 
		{ break; }			
	    } else {
		if(ucrp_passwd[0] == '\0') {	/* read password **/
		    snprintf(ucrp_passwd, sizeof(ucrp_passwd), 
		    UMSG_PS, ucrp_args[argn+1]);
	    	    argd = 2;
	        } else {		/* password file **/
		    snprintf(ucrp_pfile, sizeof(ucrp_pfile), 
		    UMSG_PS, ucrp_args[argn+1]);
	    	    argd = 2;
		}
	    }
    } else
    if(strcmp(ucrp_args[argn], "--info") == 0) {
	    ucrp_cmd |= UCRP_INFO;
    } else
    if(strcmp(ucrp_args[argn], "--hide-passwd") == 0) {
	    if(strcmp(ucrp_args[argn], "*") == 0) {
    		ucrp_hide_passwd = 1;	/* hide passwd **/
	    } else
	    if(strcmp(ucrp_args[argn], "$") == 0) {
    		ucrp_hide_passwd = 2;
	    } else
	    if(strcmp(ucrp_args[argn], "d") == 0) {
    		ucrp_hide_passwd = 3;
	    } else {
    		ucrp_hide_passwd = 1;	/* hide passwd **/
	    }
    } else
    if(strcmp(ucrp_args[argn], "--help") == 0) {
	ecmd = UCRP_HELP;		/* print usage message and exit **/
	{ break; }
    } else
    if(strcmp(ucrp_args[argn], "--license") == 0) {
	ecmd = UCRP_LIC;
	{ break; }
    } else
    if(strcmp(ucrp_args[argn], "--license-ru") == 0) {
	ecmd = UCRP_LICRU;
	{ break; }
    } else
    if(strcmp(ucrp_args[argn], "--version") == 0) {
	ecmd = UCRP_VERSION;		/* print version & exit **/
	{ break; }			
    } else
    if(strcmp(ucrp_args[argn], "--todo") == 0) {
	ecmd = UCRP_TODO;		/* print todo note & exit **/
	{ break; }
    } else
    if(strcmp(ucrp_args[argn], "--changelog") == 0) {
	ecmd = UCRP_CHANGELOG;		/* print change log & exit **/
	{ break; }
    } else
    if(strcmp(ucrp_args[argn], "--man") == 0) {
	ecmd = UCRP_MAN;		/* print man page and exit **/
	{ break; }
    } else
     {
	if(argn + 1 == ucrp_argc - 1) {
	    snprintf(ucrp_ifile, sizeof(ucrp_ifile), UMSG_PS, ucrp_args[argn]);
	    snprintf(ucrp_ofile, sizeof(ucrp_ofile), UMSG_PS, ucrp_args[argn+1]);
	    argd = 2;
	} else
	if(argn == ucrp_argc - 1) {
	    snprintf(ucrp_ifile, sizeof(ucrp_ifile), UMSG_PS, ucrp_args[argn]);
	}
    }
 }
 getcwd(ucrp_pdir, sizeof(ucrp_pdir));	/* read current working dir **/
 if(strlen(ucrp_cdir) > 0) {		 /* current dirrectory **/
    ucrp_printf("current dir:{%s}\r\n", ucrp_cdir);
    chdir(ucrp_cdir);				/* change dirrectory **/
 }
 if(ecmd == 0) {
    if((
	ucrp_cmd & 
	(
	    UCRP_ENCRYPT |		/* encrypt **/
	    UCRP_DECRYPT |		/* decrypt **/
	    UCRP_INFO |			/* file info **/
	    UCRP_CLEAR			/* clear current dir disk space & bash/mc history **/
	)) != 0
    ) {					/* print program options **/
	if(strlen(ucrp_ifile) > 0) {	/* input file name **/
	    ucrp_printf("input file:{%s}\r\n", ucrp_ifile);
	}
	if(strlen(ucrp_ofile) > 0) {	/* output file name **/
	    ucrp_printf("output file:{%s}\r\n", ucrp_ofile);
	}
	if((ucrp_src_buf_size != 0) && (ucrp_dst_buf_size != 0)) {
	    ucrp_printf(		/* print source & destination block size **/
		"src block size : %d {%x}\r\n", 
		ucrp_src_buf_size, ucrp_src_buf_size);
	    ucrp_printf(		/* print destination block size in decimal and hexadecimal mode **/
		"dst block size : %d {%x}\r\n", 
		ucrp_dst_buf_size, ucrp_dst_buf_size);
	}
	ucrp_printf("cmd:");
	if((ucrp_cmd & UCRP_ENCRYPT) != 0) {		/* encrypt passwd  **/
	    ucrp_printf("{encrypt}");
	} else 
	if((ucrp_cmd & UCRP_DECRYPT) != 0) {		/* decrypt passwd **/
	    ucrp_printf("{decrypt}");
	} else 
	if((ucrp_cmd & UCRP_INFO) != 0) {		/* file info **/
	    ucrp_printf("{info}");
	}
	if((ucrp_cmd & UCRP_CLEAR) == UCRP_CLEAR) {
	    ucrp_printf("{clear}");
	}
	ucrp_printf("\r\n");
    }
 }
 if(strlen(ucrp_passwd) > 0) {		/* not empty passwd **/
    if(ucrp_hide_passwd == 0) {		/* not hide passwd **/
	ucrp_printf("NumLock, CapsLok sensitive\r\npasswd:[%s]\r\n", ucrp_passwd);
    }
    if(strcmp(ucrp_passwd, "request") == 0) { /* passwd request **/
	int echo;			/* echo mode **/
	if(ucrp_hide_passwd > 0) {
	    switch(ucrp_hide_passwd) {
		case 1 : echo = 2; break; /* print * as echo **/
		case 2 : echo = 3; break; /* print $ as echo **/
		case 3 : echo = 4; break; /* print d as echo **/
		default: echo = 2; break;  
	    }
	} else {
	    echo = 1;			/* print entered character **/
	}
	{
	    int pos;
	    for(pos = 0; pos < (int)sizeof(ucrp_passwd); ++pos) {
		ucrp_passwd[pos] = 0;
	    }
	}
	ucrp_passwd[0] = '\0';		/* clear input buffer **/
	if(ucrp_select(			/* read passwd, echo *, wait CR **/
	    ucrp_passwd,sizeof(ucrp_passwd), echo, 1) < 0) {
	    ucrp_passwd[0] = '\0';
	}
    }
 }
 if(ecmd != 0) {
    if(ecmd == UCRP_HELP) {		/* print help usage & exit **/
	ucrp_usage(0);
    } else
    if(ecmd == UCRP_LIC) {
	ucrp_license();
    } else
    if(ecmd == UCRP_LICRU) {
	ucrp_license_ru();
    } else
    if(ecmd == UCRP_VERSION) {		/* print version & exit **/
	ucrp_version();
    } else
    if(ecmd == UCRP_TODO) {		/* print todo note & exit **/
	ucrp_todo();
    } else
    if(ecmd == UCRP_CHANGELOG) {	/* print change log & exit **/
	ucrp_changelog();
    } else
    if(ecmd == UCRP_MAN) {		/* print man page and exit **/
	ucrp_man(0);
    } else
    {
	ucrp_error("Internal error (main.c)\r\n");
    }
    exit(1);
 }

 if(strlen(ucrp_pfile) > 0) {
    ucrp_printf("pfile:(%s)\r\n", ucrp_pfile);
    ucrp_fi = fopen(ucrp_pfile, "r+b");	/* read passwd file **/
    if(ucrp_fi == 0) {
	ucrp_error("File [%s] 8 open error.\r\n", ucrp_pfile);
	return(0);
    }
    int pos;
    pos = strlen(ucrp_passwd);
    while(pos < (int) sizeof(ucrp_passwd)) {
	u_char ch;			/* readded ecc **/
	ch = fgetc(ucrp_fi);
	if(feof(ucrp_fi) != 0) {
	    break;
	}
	ucrp_passwd[pos++] = ch;
    }
    if(pos < (int)sizeof(ucrp_passwd)) {
	ucrp_passwd[pos] = '\0';
    }
    fclose(ucrp_fi);
    char * pnam;			/* last name **/
    char * pstr; 			/* temporary variable **/
    pnam = ucrp_pfile;			/* pfile **/
    pstr = pnam;			
    while(*pstr != '\0') {
	/* find string separator or string terminator **/
	while(*pstr != '\0' && *pstr != '/') {
	    pstr++;			/* next character in string **/
	}
	if(*pstr == '/') {
	/* string separator **/
	    pstr++;
	    pnam = pstr;
	}
    }
/* add file from /dbin catalog **/
    char dbin[256];			/* string buffer **/
    snprintf(				
	dbin, sizeof(dbin), 		/* string buffer & string name **/
	"/dbin/%s", pnam		/* **/
    );
    ucrp_fi = fopen(dbin, "r+b");	/* open **/
    if(ucrp_fi != 0) {
	pos = strlen(ucrp_passwd);
	while(pos < (int) sizeof(ucrp_passwd)) {
	    u_char ch;			/* readded ecc **/
	    ch = fgetc(ucrp_fi);
	    if(feof(ucrp_fi) != 0) {
		break;
	    }
	    ucrp_passwd[pos++] = ch;
	}
	if(pos < (int)sizeof(ucrp_passwd)) {
	    ucrp_passwd[pos] = '\0';
	}
	fclose(ucrp_fi);
    }
    snprintf(				/* add file from /usr/dbin catalog **/
	dbin, sizeof(dbin), 		/* string buffer & string name **/
	"/usr/dbin/%s", pnam		/* **/
    );
    ucrp_fi = fopen(dbin, "r+b");	/* open **/
    if(ucrp_fi != 0) {
	pos = strlen(ucrp_passwd);
	while(pos < (int) sizeof(ucrp_passwd)) {
	    u_char ch;			/* readded ecc **/
	    ch = fgetc(ucrp_fi);
	    if(feof(ucrp_fi) != 0) {
		break;
	    }
	    ucrp_passwd[pos++] = ch;
	}
	if(pos < (int)sizeof(ucrp_passwd)) {
	    ucrp_passwd[pos] = '\0';
	}
	fclose(ucrp_fi);
    }
 }
 if(strlen(ucrp_ifile) > 0) {
    ucrp_fi = fopen(ucrp_ifile, "r+b");
    if(ucrp_fi == 0) {
	ucrp_error("Input file [%s] 8 open error.\r\n", ucrp_ifile);
	return(0);
    }
 }
 if(strlen(ucrp_ofile) > 0) {
    ucrp_fo = fopen(ucrp_ofile, "w+b");
    if(ucrp_fo == 0) {
	ucrp_error("Input file [%s] 9 open error.\r\n", ucrp_ofile);
	return(0);
    }
 }
 if(ucrp_random_init() == 0) {		/* init random **/
    rvl = 0;
 }
 if(ucrp_comp_init() == 0) {		/* init compatibylity buffer **/
    rvl = 0;
 }
 if(ucrp_key_init() == 0) {		/* init key buffer **/
    rvl = 0;
 }
 if( ((ucrp_cmd & UCRP_INFO) != 0) || ((ucrp_cmd & UCRP_DECRYPT) != 0) ) {
    if(ucrp_comp_read() == 0) {		/* read compatibylity buffer **/
	rvl = 0;
    }
    if(ucrp_key_read() == 0) {		/* read key buffer **/
	rvl = 0;
    }
    if(ucrp_hash_init() == 0) {
	rvl = 0;
    }
    if(ucrp_comp_check() != 1) {
	ucrp_error("Not valid input file\r\n");
	ucrp_random_close();		/* close random files **/
	if(ucrp_fi != 0) {
	    fclose(ucrp_fi);
	    ucrp_fi = 0;
	} 
	if(ucrp_fo != 0) {
	    fclose(ucrp_fo);
	    ucrp_fo = 0;
	} 
	exit(0);
    }
    if(ucrp_comp_version() < 118) {
	ucrp_xor_comp = UCRP_XOR_100;	/* **/
	ucrp_xre_comp = UCRP_XRE_100;	/* **/
	ucrp_add_comp = UCRP_ADD_100;	/* **/
	ucrp_rot_comp = UCRP_ROT_100;	/* **/
	ucrp_bts_comp = UCRP_BTS_100;	/* **/
	ucrp_tab_comp = UCRP_TAB_100;	/* **/
	ucrp_ecc_comp = UCRP_ECC_100;	/* **/
	ucrp_bev_comp = UCRP_BEV_100;	/* **/
	ucrp_bts_base = UCRP_BTS_BASE_100;
	ucrp_tab_base = UCRP_TAB_BASE_100;
	ucrp_bts_table_base = UCRP_BTS_TABLE_BASE_100;
	ucrp_table_base = UCRP_TABLE_BASE_100;
	ucrp_low_bts = UCRP_LOW_BTS_100;
	ucrp_low_tab = UCRP_LOW_TAB_100;
    }
 } else
 if( ((ucrp_cmd & UCRP_ENCRYPT) != 0) ) {
    if((ucrp_cmd & UCRP_ENCRYPT) != 0) {
	ucrp_comp(ucrp_xor_comp,0);	/* encrypt -> disable XOR algorithm **/
    }
    if(ucrp_comp_write() == 0) {	/* read compatibylity buffer **/
	rvl = 0;
    }
    if(ucrp_key_write() == 0) {		/* read key buffer **/
	rvl = 0;
    }
    if(ucrp_hash_init() == 0) {
	rvl = 0;
    }
 }
 if((ucrp_src_buf_size == 0) || (ucrp_dst_buf_size == 0)) {
    ucrp_src_buf_size = UCRP_SRC_SIZE;	/* default source buffer size **/
    ucrp_dst_buf_size = UCRP_BUF_SIZE;	/* default destination buffer **/
 }
 int cval;				
 cval = 1;				/* cval **/
 if(ucrp_src == 0) {			/* buffer length **/
    int blen;				/* buffer length **/
    blen = ucrp_src_buf_size;		
    if((int) ucrp_dst_buf_size > blen) { /* max (ucrp_src_buf_size,ucrp_dst_buf_size) **/	
	blen = ucrp_dst_buf_size;
    }
    ucrp_src = (u_char *) malloc(blen);	/* alloc source buffer **/
    if(ucrp_src == 0) { 		/* memory allocation error **/
	cval = 0;
    }
 }
 if(ucrp_dst == 0) {			/* destination buffer **/
    int blen;
    blen = ucrp_src_buf_size;		/* buffer length **/
    if((int) ucrp_dst_buf_size > blen) { /* max(ucrp_src_buf_size,ucrp_dst_buf_size) **/
	blen = ucrp_dst_buf_size;
    }
    ucrp_dst = (u_char *) malloc(blen);	/* malloc destination buffer **/
    if(ucrp_dst == 0) {
	if(ucrp_src != 0) {		/* destination buffer alloc error **/
	    free(ucrp_src);		/* free source buffer **/
	    ucrp_src = 0;		/* clear source buffer **/
	}
	cval = 0;			/* clear more **/
    }
 }
 if(cval == 1) {
    if((ucrp_cmd & UCRP_INFO) != 0) {
	ucrp_info();			/* print file info and exit **/
    } else 
    if( ((ucrp_cmd & UCRP_ENCRYPT) != 0) || ((ucrp_cmd & UCRP_DECRYPT) != 0) ) {
	while(1) {			/* encrypt & decrypt file **/
	    int cnt;			/* bytes to read **/
	    cnt = ucrp_src_buf_size;	
#ifdef __UCRP_ECC_ENABLE__
	    if( ((ucrp_cmd & UCRP_DECRYPT) != 0) ) {
		if(ucrp_comp(ucrp_ecc_comp,-1) != 0) {
		    cnt = ucrp_dst_buf_size; /* ecc enabled **/
		}
	    }
#endif 							/* __UCRP_ECC_ENABLE__ **/
	    if(ucrp_src != 0) {		/* read source buffer **/
		cnt = ucrp_read(ucrp_src, cnt);
		if(cnt == 0) {			
		    break;
		}
	    }
	    typedef struct {
		char * name;		/* function name **/
		int cval;		/* compatibility **/
		int (*en)(u_char * dst, u_char * src, int cnt); /* encrypt **/
	        int (*de)(u_char * dst, u_char * src, int cnt); /* decrypt **/
	    } PROC;
	    static PROC proc100[] = {
#ifdef __UCRP_XOR_ENABLE__
		{"xor", ucrp_xor_comp, ucrp_xor_en, ucrp_xor_en}, /* XOR **/	
#endif 							 /* __UCRP_XOR_ENABLE__ **/
#ifdef __UCRP_XRE_ENABLE__
		{"xre", ucrp_xre_comp, ucrp_xre_en, ucrp_xre_en}, /* XRE **/	
#endif 							 /* __UCRP_XRE_ENABLE__ **/
#ifdef __UCRP_ADD_ENABLE__
		{"add", ucrp_add_comp, ucrp_add_en, ucrp_add_de},	/* ADD **/
#endif 							 /* __UCRP_ADD_ENABLE__ **/
#ifdef __UCRP_ROT_ENABLE__
		{"rot", ucrp_rot_comp, ucrp_rot_en, ucrp_rot_de}, 	/* ROT **/
#endif 							 /* __UCRP_ROT_ENABLE__ **/
#ifdef __UCRP_TAB_ENABLE__
		{"tab", ucrp_tab_comp, ucrp_tab_en, ucrp_tab_de}, 	/* TAB **/
#endif 							/* __UCRP_TAB_ENABLE__ **/
#ifdef __UCRP_BTS_ENABLE__
		{"bts", ucrp_bts_comp, ucrp_bts_en, ucrp_bts_de},	/* BTS **/
#endif							/* __UCRP_BTS_ENABLE__ **/
#ifdef __UCRP_ECC_ENABLE__
		{"ecc", ucrp_ecc_comp, ucrp_ecc_en, ucrp_ecc_de},	/* ECC **/
#endif 							/* __UCRP_ECC_ENABLE__ **/
		{0, 0, 0, 0},
	    }; 
	    static PROC proc118[] = {
#ifdef __UCRP_XOR_ENABLE__
		{"xor", ucrp_xor_comp, ucrp_xor_en, ucrp_xor_en}, /* XOR **/	
#endif 							 /* __UCRP_XOR_ENABLE__ **/
#ifdef __UCRP_XRE_ENABLE__
		{"xre", ucrp_xre_comp, ucrp_xre_en, ucrp_xre_en}, /* XRE **/	
#endif 							 /* __UCRP_XRE_ENABLE__ **/
#ifdef __UCRP_ADD_ENABLE__
		{"add", ucrp_add_comp, ucrp_add_en, ucrp_add_de}, /* ADD **/
#endif 							 /* __UCRP_ADD_ENABLE__ **/
#ifdef __UCRP_ROT_ENABLE__
		{"rot", ucrp_rot_comp, ucrp_rot_en, ucrp_rot_de}, /* ROT **/
#endif 							 /* __UCRP_ROT_ENABLE__ **/
#ifdef __UCRP_BTS_ENABLE__
		{"bts", ucrp_bts_comp, ucrp_bts_en, ucrp_bts_de}, /* BTS **/
#endif							/* __UCRP_BTS_ENABLE__ **/
#ifdef __UCRP_TAB_ENABLE__
		{"tab", ucrp_tab_comp, ucrp_tab_en, ucrp_tab_de}, /* TAB **/
#endif 							/* __UCRP_TAB_ENABLE__ **/
#ifdef __UCRP_ECC_ENABLE__
		{"ecc", ucrp_ecc_comp, ucrp_ecc_en, ucrp_ecc_de}, /* ECC **/
#endif 							/* __UCRP_ECC_ENABLE__ **/
		{0, 0, 0, 0},
	    }; 
	    PROC * proc;
	    if(ucrp_comp_version() >= 118) {
		proc = proc118;
	    } else {
		proc = proc100;
	    }
	    u_char * buf[2] = {
		ucrp_src,		/* source buffer **/
	        ucrp_dst		/* destination buffer **/
	    };
	    int sbn = 0;		/* source buffer number **/
	    int dbn = 1;		/* destination buffer number **/
	    int fn;			/* function number **/
	    int finc;			/* function increment value **/
	    int fcnt;			/* functions count **/
	    fcnt = 0;				/* function count **/
	    while(proc[fcnt].name != 0) {	/* calculate function count **/
		fcnt++;
	    }
	    fn = 0;
	    if((ucrp_cmd & UCRP_ENCRYPT) != 0) {
		finc = 1;
	    } else {
		while(proc[fn].name != 0) {
		    fn += 1;
		}
		fn -= 1;
		finc = -1;
	    }
	    while(fcnt-- > 0)  {

		ucrp_printf("\r\n%d %s ", /* print function name **/	
		    fn ,proc[fn].name);
		if(ucrp_comp(proc[fn].cval,-1) != 0) {
    		    ucrp_printf(".");
		    if((buf[sbn] != 0) && (buf[dbn] != 0)) {
			if((ucrp_cmd & UCRP_ENCRYPT) != 0) { /* encrypt **/
			    cnt = proc[fn].en(buf[dbn], buf[sbn], cnt);
			} else {	/* decrypt **/
			    cnt = proc[fn].de(buf[dbn], buf[sbn], cnt);
			}
			if(cnt <= 0) {
			    break;
			}
		    }
		    sbn = ((sbn + 1) & 1);	/* next source buffer **/
	    	    dbn = ((dbn + 1) & 1);	/* next destination buffer **/
		    if(cnt > 0) {
	    		ucrp_printf(" Ok");
		    } else {
	    		ucrp_printf(" Fail");
		    }
		} else {
	    	    ucrp_printf(" Skipped");
		}
		fn += finc;			/* next function **/
	    }
	    if(buf[sbn] != 0) {			/* write destination buffer **/
		ucrp_write(buf[sbn], cnt);
	    }
	    if(cnt <= 0) {
		rvl = 0;
		ucrp_error("Encryption/decryption error\r\n");
		{ break; }
	    }
	}
    }
 }
 if((ucrp_cmd & UCRP_CLEAR) != 0) {
    ucrp_proc_clear();
 }
 if(ucrp_random_close() < 0) {
    rvl = 0;
 }
 if(ucrp_fi != 0) {
    if(fclose(ucrp_fi) < 0) {
	rvl = 0;
    }
    ucrp_fi = 0;
 } 
 if(ucrp_fo != 0) {
    if(fclose(ucrp_fo) < 0) {
	rvl = 0;
    }
    ucrp_fo = 0;
 } 
 if(strlen(ucrp_pdir) > 0) {
    chdir(ucrp_pdir);			/* restore current directory **/
 }
 if(ucrp_src != 0) {
    free(ucrp_src);			/* free source buffer **/
    ucrp_src = 0;			/* clear source buffer pointer **/
 }
 if(ucrp_dst == 0) {
    free(ucrp_dst);			/* free destination buffer **/
    ucrp_dst = 0;			/* clear destination buffer pointer **/
 }
 return(rvl);
}
